package brownstrategy;

import java.util.Stack;

import brownstrategy.agent.Agent;
import strategy.UtilityBasedStrategy;
import util.Array;
import game.Game;



/**
 * The external regret matching strategy is based on Blackwell's approachability
 * theorem.
 * @author maz
 * 
 * edited by jberg
 *
 */
public class BrownExternalRegretMatchingStrategy extends BrownUtilityBasedStrategy{

	/**
	 * The total regret of not playing each algorithm.
	 */
	int[] regrets;
	int offset;
	
	Stack<int[]> regretsStack;
	Stack<Integer> offsetStack;

	/**
	 * Construct an external regret matching strategy based on
	 * Blackwell's approachability theorem.
	 * @param g
	 * @param seed
	 */
	public BrownExternalRegretMatchingStrategy(Game g, long seed, String options) {
		super(g,seed,options);
		this.regrets = new int[numActions];
		offset = 0;
		regretsStack = new Stack<int[]>();
		offsetStack = new Stack<Integer>();
	}

	public int[] getRegrets(){
		return regrets;
	}

	public void setRegrets(int[] vector){
		assert(regrets.length==vector.length);
		regrets=vector;
	}


	@Override
	public void observeUtilities(int[] possibleUtilities,int actionTaken) {
		for(int i=0;i<possibleUtilities.length;i++){
			regrets[i] += possibleUtilities[i]-possibleUtilities[actionTaken];
		}
		offset += possibleUtilities[actionTaken];
	}

	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		//System.err.println("Regret matching strategy"+Array.arrayToString(regrets));
		double total = 0;
		for(int i=0;i<regrets.length;i++){

			if (regrets[i]>0){
				total+=regrets[i];
			}
		}
		if (total>0){
			double[] adjustedRegrets = new double[regrets.length];
			for(int i=0;i<regrets.length;i++){
				adjustedRegrets[i] = regrets[i] + offset;
				if(adjustedRegrets[i] < 0) {
					adjustedRegrets[i] = 0;
				}
			}
			_cacheDist = normalize(adjustedRegrets);
			return _cacheDist;
		}
		double[] arr = new double[_g.getNumActions()];
		double uniform = 1.0/arr.length;
		for(int i = 0; i < arr.length; i++) {
			arr[i] = uniform;
		}
		_cacheDist = arr;
		return _cacheDist;
	}

	@Override
	public Agent getCopy() {
		return new BrownExternalRegretMatchingStrategy(_g, _seed, _options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new BrownExternalRegretMatchingStrategy(g, seed, options);
	}
	
	@Override
	public void lookAhead(int[] actions) {
		regretsStack.push(regrets);
		offsetStack.push(offset);
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		regrets = regretsStack.pop();
		offset = offsetStack.pop();
		_day--;
	}

}
