package brownstrategy;

import game.Game;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import util.Constants;
import brownstrategy.agent.Agent;
import brownstrategy.agent.ModelBasedAgent;
import brownstrategy.agent.NondeterministicAgent;
import brownstrategy.ensembles.AbstractEnsemble;
import brownstrategy.ensembles.BayesianInference;
import brownstrategy.ensembles.EnsembleOracle;
import brownstrategy.ensembles.PredictionByPartialMatching;
import brownstrategy.oracles.PrevSequenceAllOracle;
import brownstrategy.oracles.PrevSequenceAllRotandOppSymOracle;
import brownstrategy.oracles.PrevSequenceIndividualOracle;
import brownstrategy.oracles.PrevSequenceOppSymOracle;
import brownstrategy.oracles.PrevSequenceOpponentsOracle;
import brownstrategy.oracles.PrevSequenceZeroContextOracle;
import brownstrategy.oracles.SameAsLastTimeOracle;

/**
 *
 * @author jberg
 *
 */

public class BrownLemonade extends NondeterministicAgent {

	ArrayList<Agent> usOracles;
	ArrayList<Agent> usPlayers;

	ArrayList<Agent> opp1Oracles;
	ArrayList<AbstractEnsemble> opp1Ensembles;
	ArrayList<ModelBasedAgent> opp1ModelBasedPlayers;
	AbstractEnsemble opp1UltraEnsemble;

	ArrayList<Agent> opp2Oracles;
	ArrayList<AbstractEnsemble> opp2Ensembles;
	ArrayList<ModelBasedAgent> opp2ModelBasedPlayers;
	AbstractEnsemble opp2UltraEnsemble;

	HashMap<Agent,ArrayList<Double>> usOptimizerRegrets;
	double[] _lookahead;
	Agent agentNR;
	int _numDaysNR = 20;
	int _numSamples = 12;

	public BrownLemonade(Game g, long seed, String options) {
		super(g,seed,options);

		usOracles = new ArrayList<Agent>();
		usPlayers = new ArrayList<Agent>();

		opp1Oracles = new ArrayList<Agent>();
		opp1Ensembles = new ArrayList<AbstractEnsemble>();
		opp1ModelBasedPlayers = new ArrayList<ModelBasedAgent>();

		opp2Oracles = new ArrayList<Agent>();
		opp2Ensembles = new ArrayList<AbstractEnsemble>();
		opp2ModelBasedPlayers = new ArrayList<ModelBasedAgent>();

		ArrayList<ModelBasedAgent> usTmpModelBasedPlayers = new ArrayList<ModelBasedAgent>();
		ArrayList<ModelBasedAgent> opp1TmpModelBasedPlayers = new ArrayList<ModelBasedAgent>();
		ArrayList<ModelBasedAgent> opp2TmpModelBasedPlayers = new ArrayList<ModelBasedAgent>();

		ArrayList<Agent> usModelPredictors = new ArrayList<Agent>();
		ArrayList<Agent> opp1ModelPredictors = new ArrayList<Agent>();
		ArrayList<Agent> opp2ModelPredictors = new ArrayList<Agent>();

		usOptimizerRegrets = new HashMap<Agent, ArrayList<Double>>();
		agentNR = null;
		_lookahead = null;

		/*
		 * Initialize all of the models for each player
		 */
		Agent zeroContext = new PrevSequenceZeroContextOracle(g,seed, "0,0,false");
		usOracles.add(zeroContext);
		usOracles.add(new PrevSequenceIndividualOracle(g,seed, "0,1,false"));
		usOracles.add(new PrevSequenceIndividualOracle(g,seed, "0,2,false"));
		usOracles.add(new PrevSequenceIndividualOracle(g,seed, "0,3,false"));
		PredictionByPartialMatching PPM1 = new PredictionByPartialMatching(g, seed, "0");
		PPM1.addModel(usOracles.get(usOracles.size()-1));
		PPM1.addModel(usOracles.get(usOracles.size()-2));
		PPM1.addModel(usOracles.get(usOracles.size()-3));
		PPM1.addModel(zeroContext);
		usOracles.add(PPM1);
		usOracles.add(new PrevSequenceOppSymOracle(g,seed, "0,1,false"));
		usOracles.add(new PrevSequenceOppSymOracle(g,seed, "0,2,false"));
		usOracles.add(new PrevSequenceOppSymOracle(g,seed, "0,3,false"));
		PredictionByPartialMatching PPM2 = new PredictionByPartialMatching(g, seed, "0");
		PPM2.addModel(usOracles.get(usOracles.size()-1));
		PPM2.addModel(usOracles.get(usOracles.size()-2));
		PPM2.addModel(usOracles.get(usOracles.size()-3));
		PPM2.addModel(zeroContext);
		usOracles.add(PPM2);
		usOracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "0,1,false"));
		usOracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "0,2,false"));
		usOracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "0,3,false"));
		PredictionByPartialMatching PPM3 = new PredictionByPartialMatching(g, seed, "0");
		PPM3.addModel(usOracles.get(usOracles.size()-1));
		PPM3.addModel(usOracles.get(usOracles.size()-2));
		PPM3.addModel(usOracles.get(usOracles.size()-3));
		PPM3.addModel(zeroContext);
		usOracles.add(PPM3);

		Agent zeroContextSGT = new PrevSequenceZeroContextOracle(g,seed, "0,0,true");
		usOracles.add(zeroContextSGT);
		usOracles.add(new PrevSequenceIndividualOracle(g,seed, "0,1,true"));
		usOracles.add(new PrevSequenceIndividualOracle(g,seed, "0,2,true"));
		usOracles.add(new PrevSequenceIndividualOracle(g,seed, "0,3,true"));
		PredictionByPartialMatching PPM1SGT = new PredictionByPartialMatching(g, seed, "0");
		PPM1SGT.addModel(usOracles.get(usOracles.size()-1));
		PPM1SGT.addModel(usOracles.get(usOracles.size()-2));
		PPM1SGT.addModel(usOracles.get(usOracles.size()-3));
		PPM1SGT.addModel(zeroContextSGT);
		usOracles.add(PPM1SGT);
		usOracles.add(new PrevSequenceOppSymOracle(g,seed, "0,1,true"));
		usOracles.add(new PrevSequenceOppSymOracle(g,seed, "0,2,true"));
		usOracles.add(new PrevSequenceOppSymOracle(g,seed, "0,3,true"));
		PredictionByPartialMatching PPM2SGT = new PredictionByPartialMatching(g, seed, "0");
		PPM2SGT.addModel(usOracles.get(usOracles.size()-1));
		PPM2SGT.addModel(usOracles.get(usOracles.size()-2));
		PPM2SGT.addModel(usOracles.get(usOracles.size()-3));
		PPM2SGT.addModel(zeroContextSGT);
		usOracles.add(PPM2SGT);
		usOracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "0,1,true"));
		usOracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "0,2,true"));
		usOracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "0,3,true"));
		PredictionByPartialMatching PPM3SGT = new PredictionByPartialMatching(g, seed, "0");
		PPM3SGT.addModel(usOracles.get(usOracles.size()-1));
		PPM3SGT.addModel(usOracles.get(usOracles.size()-2));
		PPM3SGT.addModel(usOracles.get(usOracles.size()-3));
		PPM3SGT.addModel(zeroContextSGT);
		usOracles.add(PPM3SGT);

		BayesianInference bayes1 = new BayesianInference(g, seed, "0");
		bayes1.addModel(PPM1);
		bayes1.addModel(PPM1SGT);
		usModelPredictors.add(bayes1);
		usOracles.add(bayes1);

		BayesianInference bayes2 = new BayesianInference(g, seed, "0");
		bayes2.addModel(PPM2);
		bayes2.addModel(PPM2SGT);
		usModelPredictors.add(bayes2);
		usOracles.add(bayes2);

		BayesianInference bayes3 = new BayesianInference(g, seed, "0");
		bayes3.addModel(PPM3);
		bayes3.addModel(PPM3SGT);
		usModelPredictors.add(bayes3);
		usOracles.add(bayes3);

		usTmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"0,0,2"));

		ArrayList<Agent> tmpOpp1Ensemble = new ArrayList<Agent>();
		Agent zeroContext1 = new PrevSequenceZeroContextOracle(g,seed, "1,0,false");
		opp1Oracles.add(zeroContext1);
		opp1Oracles.add(new PrevSequenceIndividualOracle(g,seed, "1,1,false"));
		opp1Oracles.add(new PrevSequenceIndividualOracle(g,seed, "1,2,false"));
		opp1Oracles.add(new PrevSequenceIndividualOracle(g,seed, "1,3,false"));
		PredictionByPartialMatching PPM4 = new PredictionByPartialMatching(g, seed, "1");
		PPM4.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM4.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM4.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM4.addModel(zeroContext1);
		tmpOpp1Ensemble.add(PPM4);
		opp1Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "1,1,false"));
		opp1Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "1,2,false"));
		opp1Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "1,3,false"));
		PredictionByPartialMatching PPM5 = new PredictionByPartialMatching(g, seed, "1");
		PPM5.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM5.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM5.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM5.addModel(zeroContext1);
		tmpOpp1Ensemble.add(PPM5);
		opp1Oracles.add(new PrevSequenceOppSymOracle(g,seed, "1,1,false"));
		opp1Oracles.add(new PrevSequenceOppSymOracle(g,seed, "1,2,false"));
		opp1Oracles.add(new PrevSequenceOppSymOracle(g,seed, "1,3,false"));
		PredictionByPartialMatching PPM6 = new PredictionByPartialMatching(g, seed, "1");
		PPM6.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM6.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM6.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM6.addModel(zeroContext1);
		tmpOpp1Ensemble.add(PPM6);
		opp1Oracles.add(new PrevSequenceAllOracle(g,seed, "1,1,false"));
		opp1Oracles.add(new PrevSequenceAllOracle(g,seed, "1,2,false"));
		opp1Oracles.add(new PrevSequenceAllOracle(g,seed, "1,3,false"));
		PredictionByPartialMatching PPM7 = new PredictionByPartialMatching(g, seed, "1");
		PPM7.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM7.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM7.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM7.addModel(zeroContext1);
		tmpOpp1Ensemble.add(PPM7);
		opp1Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "1,1,false"));
		opp1Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "1,2,false"));
		opp1Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "1,3,false"));
		PredictionByPartialMatching PPM8 = new PredictionByPartialMatching(g, seed, "1");
		PPM8.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM8.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM8.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM8.addModel(zeroContext1);
		tmpOpp1Ensemble.add(PPM8);


		Agent zeroContext1SGT = new PrevSequenceZeroContextOracle(g,seed, "1,0,true");
		opp1Oracles.add(zeroContext1SGT);
		opp1Oracles.add(new PrevSequenceIndividualOracle(g,seed, "1,1,true"));
		opp1Oracles.add(new PrevSequenceIndividualOracle(g,seed, "1,2,true"));
		opp1Oracles.add(new PrevSequenceIndividualOracle(g,seed, "1,3,true"));
		PredictionByPartialMatching PPM4SGT = new PredictionByPartialMatching(g, seed, "1");
		PPM4SGT.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM4SGT.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM4SGT.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM4SGT.addModel(zeroContext1SGT);
		tmpOpp1Ensemble.add(PPM4SGT);
		opp1Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "1,1,true"));
		opp1Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "1,2,true"));
		opp1Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "1,3,true"));
		PredictionByPartialMatching PPM5SGT = new PredictionByPartialMatching(g, seed, "1");
		PPM5SGT.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM5SGT.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM5SGT.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM5SGT.addModel(zeroContext1SGT);
		tmpOpp1Ensemble.add(PPM5SGT);
		opp1Oracles.add(new PrevSequenceOppSymOracle(g,seed, "1,1,true"));
		opp1Oracles.add(new PrevSequenceOppSymOracle(g,seed, "1,2,true"));
		opp1Oracles.add(new PrevSequenceOppSymOracle(g,seed, "1,3,true"));
		PredictionByPartialMatching PPM6SGT = new PredictionByPartialMatching(g, seed, "1");
		PPM6SGT.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM6SGT.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM6SGT.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM6SGT.addModel(zeroContext1SGT);
		tmpOpp1Ensemble.add(PPM6SGT);
		opp1Oracles.add(new PrevSequenceAllOracle(g,seed, "1,1,true"));
		opp1Oracles.add(new PrevSequenceAllOracle(g,seed, "1,2,true"));
		opp1Oracles.add(new PrevSequenceAllOracle(g,seed, "1,3,true"));
		PredictionByPartialMatching PPM7SGT = new PredictionByPartialMatching(g, seed, "1");
		PPM7SGT.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM7SGT.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM7SGT.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM7SGT.addModel(zeroContext1SGT);
		tmpOpp1Ensemble.add(PPM7SGT);
		opp1Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "1,1,true"));
		opp1Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "1,2,true"));
		opp1Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "1,3,true"));
		PredictionByPartialMatching PPM8SGT = new PredictionByPartialMatching(g, seed, "1");
		PPM8SGT.addModel(opp1Oracles.get(opp1Oracles.size()-1));
		PPM8SGT.addModel(opp1Oracles.get(opp1Oracles.size()-2));
		PPM8SGT.addModel(opp1Oracles.get(opp1Oracles.size()-3));
		PPM8SGT.addModel(zeroContext1SGT);
		tmpOpp1Ensemble.add(PPM8SGT);


		BayesianInference bayes4 = new BayesianInference(g, seed, "1");
		bayes4.addModel(PPM4);
		bayes4.addModel(PPM4SGT);
		opp1ModelPredictors.add(bayes4);
		tmpOpp1Ensemble.add(bayes4);

		BayesianInference bayes5 = new BayesianInference(g, seed, "1");
		bayes5.addModel(PPM5);
		bayes5.addModel(PPM5SGT);
		//		opp1ModelPredictors.add(bayes5);
		tmpOpp1Ensemble.add(bayes5);

		BayesianInference bayes6 = new BayesianInference(g, seed, "1");
		bayes6.addModel(PPM6);
		bayes6.addModel(PPM6SGT);
		opp1ModelPredictors.add(bayes6);
		tmpOpp1Ensemble.add(bayes6);

		BayesianInference bayes7 = new BayesianInference(g, seed, "1");
		bayes7.addModel(PPM7);
		bayes7.addModel(PPM7SGT);
		//		opp1ModelPredictors.add(bayes7);
		tmpOpp1Ensemble.add(bayes7);

		BayesianInference bayes8 = new BayesianInference(g, seed, "1");
		bayes8.addModel(PPM8);
		bayes8.addModel(PPM8SGT);
		opp1ModelPredictors.add(bayes8);
		tmpOpp1Ensemble.add(bayes8);

		opp1Oracles.add(new MultiplyAllStrategy(g,seed, "1"));
		opp1Oracles.add(new MultiplyOppStrategy(g,seed, "1"));
		opp1Oracles.add(new BrownExternalRegretMatchingStrategy(g,seed, "1"));
		opp1Oracles.add(new BrownGIGA(g,seed, "1"));
		opp1Ensembles.add(new EnsembleOracle(g, seed, "1,0,1.0,30,1"));
		opp1Ensembles.add(new EnsembleOracle(g, seed, "1,0,1.0,30,5"));
		opp1Ensembles.add(new EnsembleOracle(g, seed, "1,0,1.0,15,1"));
		opp1Ensembles.add(new EnsembleOracle(g, seed, "1,0,1.0,15,5"));
		opp1Ensembles.add(new EnsembleOracle(g, seed, "1,0,1.0,5,1"));
		opp1Ensembles.add(new EnsembleOracle(g, seed, "1,0,1.0,5,5"));
		opp1Ensembles.add(new BayesianInference(g,seed, "1"));
		opp1TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"1,0,2"));
		opp1TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"1,1,0"));
		opp1TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"1,1,1"));
		opp1TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"1,1,2"));
		opp1UltraEnsemble = new EnsembleOracle(g, seed, "1,0,1.0,7,1");

		ArrayList<Agent> tmpOpp2Ensemble = new ArrayList<Agent>();
		Agent zeroContext2 = new PrevSequenceZeroContextOracle(g,seed, "2,0,false");
		opp2Oracles.add(zeroContext2);
		opp2Oracles.add(new PrevSequenceIndividualOracle(g,seed, "2,1,false"));
		opp2Oracles.add(new PrevSequenceIndividualOracle(g,seed, "2,2,false"));
		opp2Oracles.add(new PrevSequenceIndividualOracle(g,seed, "2,3,false"));
		PredictionByPartialMatching PPM9 = new PredictionByPartialMatching(g, seed, "2");
		PPM9.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM9.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM9.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM9.addModel(zeroContext2);
		tmpOpp2Ensemble.add(PPM9);
		opp2Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "2,1,false"));
		opp2Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "2,2,false"));
		opp2Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "2,3,false"));
		PredictionByPartialMatching PPM10 = new PredictionByPartialMatching(g, seed, "2");
		PPM10.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM10.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM10.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM10.addModel(zeroContext2);
		tmpOpp2Ensemble.add(PPM10);
		opp2Oracles.add(new PrevSequenceOppSymOracle(g,seed, "2,1,false"));
		opp2Oracles.add(new PrevSequenceOppSymOracle(g,seed, "2,2,false"));
		opp2Oracles.add(new PrevSequenceOppSymOracle(g,seed, "2,3,false"));
		PredictionByPartialMatching PPM11 = new PredictionByPartialMatching(g, seed, "2");
		PPM11.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM11.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM11.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM11.addModel(zeroContext2);
		tmpOpp2Ensemble.add(PPM11);
		opp2Oracles.add(new PrevSequenceAllOracle(g,seed, "2,1,false"));
		opp2Oracles.add(new PrevSequenceAllOracle(g,seed, "2,2,false"));
		opp2Oracles.add(new PrevSequenceAllOracle(g,seed, "2,3,false"));
		PredictionByPartialMatching PPM12 = new PredictionByPartialMatching(g, seed, "2");
		PPM12.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM12.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM12.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM12.addModel(zeroContext2);
		tmpOpp2Ensemble.add(PPM12);
		opp2Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "2,1,false"));
		opp2Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "2,2,false"));
		opp2Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "2,3,false"));
		PredictionByPartialMatching PPM13 = new PredictionByPartialMatching(g, seed, "2");
		PPM13.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM13.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM13.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM13.addModel(zeroContext2);
		tmpOpp2Ensemble.add(PPM13);

		Agent zeroContext2SGT = new PrevSequenceZeroContextOracle(g,seed, "2,0,true");
		opp2Oracles.add(zeroContext2SGT);
		opp2Oracles.add(new PrevSequenceIndividualOracle(g,seed, "2,1,true"));
		opp2Oracles.add(new PrevSequenceIndividualOracle(g,seed, "2,2,true"));
		opp2Oracles.add(new PrevSequenceIndividualOracle(g,seed, "2,3,true"));
		PredictionByPartialMatching PPM9SGT = new PredictionByPartialMatching(g, seed, "2");
		PPM9SGT.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM9SGT.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM9SGT.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM9SGT.addModel(zeroContext2SGT);
		tmpOpp2Ensemble.add(PPM9SGT);
		opp2Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "2,1,true"));
		opp2Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "2,2,true"));
		opp2Oracles.add(new PrevSequenceOpponentsOracle(g,seed, "2,3,true"));
		PredictionByPartialMatching PPM10SGT = new PredictionByPartialMatching(g, seed, "2");
		PPM10SGT.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM10SGT.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM10SGT.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM10SGT.addModel(zeroContext2SGT);
		tmpOpp2Ensemble.add(PPM10SGT);
		opp2Oracles.add(new PrevSequenceOppSymOracle(g,seed, "2,1,true"));
		opp2Oracles.add(new PrevSequenceOppSymOracle(g,seed, "2,2,true"));
		opp2Oracles.add(new PrevSequenceOppSymOracle(g,seed, "2,3,true"));
		PredictionByPartialMatching PPM11SGT = new PredictionByPartialMatching(g, seed, "2");
		PPM11SGT.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM11SGT.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM11SGT.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM11SGT.addModel(zeroContext2SGT);
		tmpOpp2Ensemble.add(PPM11SGT);
		opp2Oracles.add(new PrevSequenceAllOracle(g,seed, "2,1,true"));
		opp2Oracles.add(new PrevSequenceAllOracle(g,seed, "2,2,true"));
		opp2Oracles.add(new PrevSequenceAllOracle(g,seed, "2,3,true"));
		PredictionByPartialMatching PPM12SGT = new PredictionByPartialMatching(g, seed, "2");
		PPM12SGT.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM12SGT.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM12SGT.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM12SGT.addModel(zeroContext2SGT);
		tmpOpp2Ensemble.add(PPM12SGT);
		opp2Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "2,1,true"));
		opp2Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "2,2,true"));
		opp2Oracles.add(new PrevSequenceAllRotandOppSymOracle(g,seed, "2,3,true"));
		PredictionByPartialMatching PPM13SGT = new PredictionByPartialMatching(g, seed, "2");
		PPM13SGT.addModel(opp2Oracles.get(opp2Oracles.size()-1));
		PPM13SGT.addModel(opp2Oracles.get(opp2Oracles.size()-2));
		PPM13SGT.addModel(opp2Oracles.get(opp2Oracles.size()-3));
		PPM13SGT.addModel(zeroContext2SGT);
		tmpOpp2Ensemble.add(PPM13SGT);


		BayesianInference bayes9 = new BayesianInference(g, seed, "2");
		bayes9.addModel(PPM9);
		bayes9.addModel(PPM9SGT);
		opp2ModelPredictors.add(bayes9);
		tmpOpp2Ensemble.add(bayes9);

		BayesianInference bayes10 = new BayesianInference(g, seed, "2");
		bayes10.addModel(PPM10);
		bayes10.addModel(PPM10SGT);
		//		opp2ModelPredictors.add(bayes10);
		tmpOpp2Ensemble.add(bayes10);

		BayesianInference bayes11 = new BayesianInference(g, seed, "2");
		bayes11.addModel(PPM11);
		bayes11.addModel(PPM11SGT);
		opp2ModelPredictors.add(bayes11);
		tmpOpp2Ensemble.add(bayes11);

		BayesianInference bayes12 = new BayesianInference(g, seed, "2");
		bayes12.addModel(PPM12);
		bayes12.addModel(PPM12SGT);
		//		opp2ModelPredictors.add(bayes12);
		tmpOpp2Ensemble.add(bayes12);

		BayesianInference bayes13 = new BayesianInference(g, seed, "2");
		bayes13.addModel(PPM13);
		bayes13.addModel(PPM13SGT);
		opp2ModelPredictors.add(bayes13);
		tmpOpp2Ensemble.add(bayes13);

		opp2Oracles.add(new MultiplyAllStrategy(g,seed, "2"));
		opp2Oracles.add(new MultiplyOppStrategy(g,seed, "2"));
		opp2Oracles.add(new BrownExternalRegretMatchingStrategy(g,seed, "2"));
		opp2Oracles.add(new BrownGIGA(g,seed, "2"));
		opp2Ensembles.add(new EnsembleOracle(g, seed, "2,0,1.0,30,1"));
		opp2Ensembles.add(new EnsembleOracle(g, seed, "2,0,1.0,30,5"));
		opp2Ensembles.add(new EnsembleOracle(g, seed, "2,0,1.0,15,1"));
		opp2Ensembles.add(new EnsembleOracle(g, seed, "2,0,1.0,15,5"));
		opp2Ensembles.add(new EnsembleOracle(g, seed, "2,0,1.0,5,1"));
		opp2Ensembles.add(new EnsembleOracle(g, seed, "2,0,1.0,5,5"));
		opp2Ensembles.add(new BayesianInference(g,seed, "2"));
		opp2TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"2,0,2"));
		opp2TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"2,1,0"));
		opp2TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"2,1,1"));
		opp2TmpModelBasedPlayers.add(new SimpleBestResponse(g,seed,"2,1,2"));
		opp2UltraEnsemble = new EnsembleOracle(g, seed, "2,0,1.0,7,1");

		/*
		 * Set up the model based agents
		 * 
		 * We need one for every combination of models
		 */
		for(int i = 0; i < opp1TmpModelBasedPlayers.size(); i++) {
			for(Agent oracle1 : usModelPredictors) {
				for(Agent oracle2 : opp2ModelPredictors) {
					ModelBasedAgent tmp = (ModelBasedAgent) opp1TmpModelBasedPlayers.get(i).getCopy();
					tmp.setPredictors(oracle1, oracle2);
					opp1ModelBasedPlayers.add(tmp);
				}
			}
		}

		for(int i = 0; i < opp2TmpModelBasedPlayers.size(); i++) {
			for(Agent oracle1 : usModelPredictors) {
				for(Agent oracle2 : opp1ModelPredictors) {
					ModelBasedAgent tmp = (ModelBasedAgent) opp2TmpModelBasedPlayers.get(i).getCopy();
					tmp.setPredictors(oracle1, oracle2);
					opp2ModelBasedPlayers.add(tmp);
				}
			}
		}

		/*
		 * Set up the ensembles
		 */
		ArrayList<Agent> opp1tmpList = new ArrayList<Agent>();
		opp1tmpList.addAll(opp1Oracles);
		opp1tmpList.addAll(opp1ModelBasedPlayers);
		for(AbstractEnsemble ensemble : opp1Ensembles) {
			ensemble.addAllModels(opp1tmpList);
		}
		ArrayList<Agent> opp1tmpEnsembleList = new ArrayList<Agent>();
		for(Agent agent : tmpOpp1Ensemble) {
			opp1Ensembles.add((AbstractEnsemble)agent);
		}
		opp1tmpEnsembleList.addAll(opp1Ensembles);
		opp1UltraEnsemble.addAllModels(opp1tmpEnsembleList);

		ArrayList<Agent> opp2tmpList = new ArrayList<Agent>();
		opp2tmpList.addAll(opp2Oracles);
		opp2tmpList.addAll(opp2ModelBasedPlayers);
		for(AbstractEnsemble ensemble : opp2Ensembles) {
			ensemble.addAllModels(opp2tmpList);
		}
		ArrayList<Agent> opp2tmpEnsembleList = new ArrayList<Agent>();
		for(Agent agent : tmpOpp2Ensemble) {
			opp2Ensembles.add((AbstractEnsemble)agent);
		}
		opp2tmpEnsembleList.addAll(opp2Ensembles);
		opp2UltraEnsemble.addAllModels(opp2tmpEnsembleList);

		for(int i = 0; i < usTmpModelBasedPlayers.size(); i++) {
			ModelBasedAgent tmp = (ModelBasedAgent) usTmpModelBasedPlayers.get(i).getCopy();
			tmp.setPredictors(opp1UltraEnsemble, opp2UltraEnsemble);
			usPlayers.add(tmp);
		}

		usPlayers.add(new BrownExternalRegretMatchingStrategy(g,seed, "0"));
		usPlayers.add(new BrownGIGA(g,seed, "0"));

		usPlayers.add(new DeBruijn2(g,seed, "0"));
		usPlayers.add(new DeBruijn3(g,seed, "0"));
		try {
			usPlayers.add(new DeBruijn4(g,seed, "0"));
		} catch (IOException e) {
		}

		for(int i = 0; i < 12; i++) {
			usPlayers.add(new BrownConstantStrategy(g,seed, "0," + i));
		}

		for(Agent agent : usPlayers) {
			ArrayList<Double> regrets = new ArrayList<Double>();
			usOptimizerRegrets.put(agent, regrets);
		}
		ArrayList<Double> regrets = new ArrayList<Double>();
		usOptimizerRegrets.put(this, regrets);
	}

	@Override
	public void observeOutcome(int[] actions) {
		/*
		 * Get player moves
		 */
		HashMap<Agent,double[]> lastPredictions = new HashMap<Agent,double[]>();
		for(Agent oracle : usPlayers) {
			double[] dist;
			dist = oracle.getActionDistribution();
			lastPredictions.put(oracle,dist);
		}

		/*
		 * Rank all strategies using NR algo and
		 * yesterdays ensembles
		 */

		int[] possibleUtils = new int[_g.getNumActions()];
		for(int i = 0; i < possibleUtils.length; i++) {
			possibleUtils[i] = Constants.payoffs[i][actions[1]][actions[2]][0];
		}
		for(Agent oracle : lastPredictions.keySet()) {
			double[] dist = lastPredictions.get(oracle);
			if(dist != null) {
				double sum = 0;
				for(int i = 0; i < dist.length; i++) {
					sum += dist[i] * possibleUtils[i];
				}
				ArrayList<Double> regrets = usOptimizerRegrets.get(oracle);
				regrets.add(sum);
				usOptimizerRegrets.put(oracle, regrets);
			}
		}
		
		if(_lookahead != null) {
			double sum = 0;
			for(int i = 0; i < _lookahead.length; i++) {
				sum += _lookahead[i] * possibleUtils[i];
			}
			ArrayList<Double> regrets = usOptimizerRegrets.get(this);
			regrets.add(sum*1.1);
			usOptimizerRegrets.put(this, regrets);
		}

		/*
		 * update ensembles
		 */
		int[] moves1 = actions.clone();

		for(Agent oracle : usOracles) {
			oracle.observeOutcome(moves1);
		}
		for(Agent oracle : opp1Oracles) {
			oracle.observeOutcome(moves1);
		}
		for(Agent oracle : opp2Oracles) {
			oracle.observeOutcome(moves1);
		}
		for(Agent oracle : opp1Ensembles) {
			oracle.observeOutcome(moves1);
		}
		for(Agent oracle : opp1ModelBasedPlayers) {
			oracle.observeOutcome(moves1);
		}
		for(Agent oracle : opp2ModelBasedPlayers) {
			oracle.observeOutcome(moves1);
		}
		opp1UltraEnsemble.observeOutcome(moves1);
		for(Agent oracle : opp2Ensembles) {
			oracle.observeOutcome(moves1);
		}
		opp2UltraEnsemble.observeOutcome(moves1);
		for(Agent oracle : usPlayers) {
			oracle.observeOutcome(moves1);
		}
	}

	@Override
	public double[] getActionDistribution() {
		double[] opp1Dist = opp1UltraEnsemble.getActionDistribution();
		double[] opp2Dist = opp2UltraEnsemble.getActionDistribution();
		double[] opp1CumDist = cumulativeSum( opp1Dist );
		double[] opp2CumDist = cumulativeSum( opp2Dist );

		int opp1Action = 0;
		int opp2Action = 0;

		double[][] expectedScoresForPlayers = getExpectedScoresGivenActionDistributions(opp1Dist, opp2Dist);
		ArrayList<Integer> bestActionList = null;
		double bestActionUtil = -Double.MAX_VALUE;
		for(int i = 0; i < _g.getNumActions(); i++) {
			int[] utils = new int[_g.getNumActions()];
			for(int samp = 0; samp < _numSamples; samp++) {
				double val1 = random.nextDouble();	
				for (int j=0; j<Constants.NUM_CLOCK_POSITIONS; j++) {
					if (val1 < opp1CumDist[j]) {
						opp1Action = j;
						break;
					}
				}
				double val2 = random.nextDouble();	
				for (int j=0; j<Constants.NUM_CLOCK_POSITIONS; j++) {
					if (val2 < opp2CumDist[j]) {
						opp2Action = j;
						break;
					}
				}

				/*
				 * Push all of the predictors forwards
				 */
				int[] newactions = new int[3];
				newactions[0] = i;
				newactions[1] = opp1Action;
				newactions[2] = opp2Action;

				/*
				 * update ensembles
				 */
				int[] moves1 = newactions.clone();

				for(Agent oracle : usOracles) {
					oracle.lookAhead(moves1);
				}
				for(Agent oracle : opp1Oracles) {
					oracle.lookAhead(moves1);
				}
				for(Agent oracle : opp2Oracles) {
					oracle.lookAhead(moves1);
				}
				for(Agent oracle : opp1Ensembles) {
					oracle.lookAhead(moves1);
				}
				for(Agent oracle : opp1ModelBasedPlayers) {
					oracle.lookAhead(moves1);
				}
				for(Agent oracle : opp2ModelBasedPlayers) {
					oracle.lookAhead(moves1);
				}
				opp1UltraEnsemble.lookAhead(moves1);
				for(Agent oracle : opp2Ensembles) {
					oracle.lookAhead(moves1);
				}
				opp2UltraEnsemble.lookAhead(moves1);
				for(Agent oracle : usPlayers) {
					oracle.lookAhead(moves1);
				}

				/*
				 * Evaluate Utility of Each Move
				 */
				double[][] expScoresLookAhead = getExpectedScoresGivenActionDistributions(opp1UltraEnsemble.getActionDistribution(), opp2UltraEnsemble.getActionDistribution());

				for (int action=0; action<Constants.NUM_CLOCK_POSITIONS; action++) {
					utils[action] += expScoresLookAhead[action][0];
				}

				/*
				 * Push all of the predictors backwards
				 */
				for(Agent oracle : usOracles) {
					oracle.lookBack();
				}
				for(Agent oracle : opp1Oracles) {
					oracle.lookBack();
				}
				for(Agent oracle : opp2Oracles) {
					oracle.lookBack();
				}
				for(Agent oracle : opp1Ensembles) {
					oracle.lookBack();
				}
				for(Agent oracle : opp1ModelBasedPlayers) {
					oracle.lookBack();
				}
				for(Agent oracle : opp2ModelBasedPlayers) {
					oracle.lookBack();
				}
				opp1UltraEnsemble.lookBack();
				for(Agent oracle : opp2Ensembles) {
					oracle.lookBack();
				}
				opp2UltraEnsemble.lookBack();
				for(Agent oracle : usPlayers) {
					oracle.lookBack();
				}
			}
			/*
			 * pick the best move and compare to best sequence thus far
			 */
			int bestScore = 0;
			for (int action=0; action<Constants.NUM_CLOCK_POSITIONS; action++) {
				if (utils[action] > bestScore) {
					bestScore = utils[action];
				}
			}

			double totScore = bestScore/((double)_numSamples) + expectedScoresForPlayers[i][0];

			if(totScore > bestActionUtil) {
				bestActionUtil = totScore;
				bestActionList = new ArrayList<Integer>();
				bestActionList.add(i);
			}
			else if(totScore == bestActionUtil) {
				bestActionList.add(i);
			}
		}
		double[] lookahead = new double[_g.getNumActions()];
		for(int i = 0; i < bestActionList.size(); i++) {
			lookahead[bestActionList.get(i)] = 1.0/bestActionList.size();
		}
		_lookahead = lookahead;

		Agent mostRegretAgent = null;
		double mostRegret = -Double.MAX_VALUE;
		for (Agent agent : usOptimizerRegrets.keySet()) {
			double regret = 0;
			ArrayList<Double> regrets = usOptimizerRegrets.get(agent);
			while(regrets.size() > _numDaysNR) {
				regrets.remove(0);
			}
			for(int i = 0; i < _numDaysNR && i < regrets.size(); i++) {
				regret += regrets.get(regrets.size()-1-i);
			}
			if(regret > mostRegret) {
				mostRegretAgent = agent;
				mostRegret = regret;
			}
		}
		if(mostRegretAgent == null) {
			return usPlayers.get(random.nextInt(usPlayers.size())).getActionDistribution();
		}
		else if(mostRegretAgent == this) {
			return lookahead;
		}
		else {
			agentNR = mostRegretAgent;
			return agentNR.getActionDistribution();
		}
	}


	public String toString() {
		return "BrownLemonade (numdaysNR: " + _numDaysNR + ", numSamps: " + _numSamples;
	}

	public BrownLemonade getCopy() {
		return new BrownLemonade(_g,_seed,_options);
	}

	@Override
	public BrownLemonade getCopy(Game g, long seed, String options) {
		return new BrownLemonade(g,seed,options);
	}

	@Override
	public void lookAhead(int[] actions) {
		//nothing to do
	}

	@Override
	public void lookBack() {
		//nothing to do
	}

}