package brownstrategy;

import game.Game;
import game.Strategy;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Random;
import java.util.StringTokenizer;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import brownstrategy.agent.Agent;
import brownstrategy.agent.DeterministicAgent;
import brownstrategy.agent.NondeterministicAgent;

/**
 * 
 * This contains every length 3 subsequence of moves
 * possible.  It is called a de bruijn sequence.
 * This agent picks a random starting point in the
 * de bruijn sequence and just plays through.
 * 
 * @author jberg
 *
 */
public class DeBruijn4 extends NondeterministicAgent {
	/**
	 * The number of actions available in the game.
	 */
	int numActions;

	/**
	 * The last actions that were observed
	 */
	int[] lastActions;

	/**
	 * Number of rounds thus far
	 */
	int numRounds;

	/**
	 * Random starting point in the de bruijn sequence
	 */
	int deBruijnStart;

	ArrayList<Integer> deBruijn;
	
	/**
	 * A random number generator for the random actions.
	 */
	Random r;
	public DeBruijn4(Game g, long seed, String options) throws IOException {
		super(g,seed,options);
		this.numActions = g.getNumActions();
		this.lastActions = null;
		this.r = new Random(seed);
		numRounds = 0;
		deBruijn = new ArrayList<Integer>();
		
		JarFile file = new JarFile("brown.jar");
		ZipEntry entry = file.getEntry("brownstrategy/debruijn4.csv");
		if (entry==null){
			throw new IOException("File not found");
		}
	  InputStream is = file.getInputStream(entry);
		BufferedReader bufRdr = new BufferedReader(new InputStreamReader(is));
		//File file = new File("brownstrategy/debruijn4.csv");
		//BufferedReader bufRdr  = new BufferedReader(new FileReader(file));
		String line = null;
		while((line = bufRdr.readLine()) != null) {
			StringTokenizer st = new StringTokenizer(line,",");
			while (st.hasMoreTokens()) {
				deBruijn.add(Integer.parseInt(st.nextToken()));
			}
		}
		bufRdr.close();
		deBruijnStart = r.nextInt(deBruijn.size());
	}

	@Override
	public Agent getCopy() {
		try {
			return new DeBruijn4(_g, _seed, _options);
		} catch (IOException e) {
			return new DeBruijn3(_g, _seed, _options);
		}
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		try {
			return new DeBruijn4(g, seed, options);
		} catch (IOException e) {
			return new DeBruijn3(g, seed, options);
		}
	}

	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		int action = deBruijn.get((deBruijnStart + numRounds) % deBruijn.size());
		_cacheDist[action] = 1;
		return _cacheDist;
	}

	@Override
	public void observeOutcome(int[] actions) {
		lastActions = actions;
		_day++;
		numRounds++;
	}
	
	@Override
	public void lookAhead(int[] actions) {
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		_day--;
		numRounds--;
	}
	
}
