package brownstrategy.ensembles;

import game.Game;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;

import brownstrategy.agent.Agent;
import brownstrategy.agent.NondeterministicAgent;
import brownstrategy.oracles.PrevSeqIndividualWithUtilOracle;
import brownstrategy.oracles.PrevSeqOpponentsWithUtilOracle;
import brownstrategy.oracles.PrevSequenceAllOracle;
import brownstrategy.oracles.PrevSequenceIndividualOracle;
import brownstrategy.oracles.PrevSequenceAllRotSymOracle;
import brownstrategy.oracles.PrevSequenceOppSymOracle;
import brownstrategy.oracles.PrevSequenceOpponentsOracle;
import brownstrategy.oracles.SameAsLastTimeOracle;

/**
 *
 *	This is an ensemble class of oracles
 * 
 * @author jberg
 *
 */

public class BayesianInference extends AbstractEnsemble {

	public BayesianInference(Game g, long seed, String options) {
		super(g,seed,options);
	}

	@Override
	public void observeOutcome(int[] actions) {
		/*
		 * Update Weights
		 */
		int action = actions[_playerIdx];
		double sum = 0;
		for(Agent agent : _lastPredictions.keySet()) {
			double[] lastPrediction = _lastPredictions.get(agent);
			if(lastPrediction != null) {
				sum += _weights.get(agent) * lastPrediction[action];
			}
		}
		
		if(sum == 0) {
			for(Agent agent : _lastPredictions.keySet()) {
				_weights.put(agent, 1.0/_lastPredictions.size());
			}
		}
		else {
			double weightsum = 0;
			for(Agent agent : _lastPredictions.keySet()) {
				double[] lastPrediction = _lastPredictions.get(agent);
				double weight;
				if(lastPrediction != null) {
					weight = _weights.get(agent) * (lastPrediction[action]/sum);
				}
				else {
					weight = 0;
				}
				weight = Math.max(0.1/_lastPredictions.size(), Math.min(0.99,weight));
				_weights.put(agent,weight);
				weightsum += weight;
			}

			for(Agent agent : _lastPredictions.keySet()) {
				_weights.put(agent,_weights.get(agent)/weightsum);
			}
		}
		
		/*
		 * Get new predictions
		 */
		for(Agent agent : _lastPredictions.keySet()) {
			_lastPredictions.put(agent, agent.getActionDistribution());
		}
	}

	public BayesianInference getCopy() {
		return new BayesianInference(_g,_seed,_options);
	}

	public String toString() {
		return "Bayesian Inference";
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new BayesianInference(g,seed,options);
	}
}