package brownstrategy.ensembles;

import game.Game;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;

import brownstrategy.agent.Agent;
import brownstrategy.agent.NondeterministicAgent;
import brownstrategy.oracles.AbstractPrevSequenceOracle;
import brownstrategy.oracles.PrevSeqIndividualWithUtilOracle;
import brownstrategy.oracles.PrevSeqOpponentsWithUtilOracle;
import brownstrategy.oracles.PrevSequenceAllOracle;
import brownstrategy.oracles.PrevSequenceIndividualOracle;
import brownstrategy.oracles.PrevSequenceAllRotSymOracle;
import brownstrategy.oracles.PrevSequenceOppSymOracle;
import brownstrategy.oracles.PrevSequenceOpponentsOracle;
import brownstrategy.oracles.SameAsLastTimeOracle;

/**
 *
 *	This is an ensemble class of oracles
 * 
 * @author jberg
 *
 */

public class PredictionByPartialMatching extends AbstractEnsemble {

	public PredictionByPartialMatching(Game g, long seed, String options) {
		super(g,seed,options);
	}

	@Override
	public void observeOutcome(int[] actions) {
		Agent oracle = null;
		for(int i = 0; i < _agentList.size(); i++) {
			if(_agentList.get(i) instanceof AbstractPrevSequenceOracle) {
				AbstractPrevSequenceOracle agent = (AbstractPrevSequenceOracle) _agentList.get(i);
				int[] oppDist = agent.getOppDist(_playerIdx);
				int total = 0;
				for(int j = 0; j < oppDist.length; j++) {
					total += oppDist[j];
				}
				if(total >= 3) {
					oracle = agent;
					break;
				}
			}
		}
		if(oracle == null) {
			oracle = _agentList.get(_agentList.size()-1);
		}
		
		for(Agent agent : _weights.keySet()) {
			if(agent == oracle) {
				_weights.put(agent, 1.0);
			}
			else {
				_weights.put(agent, 0.0);
			}
		}

		/*
		 * Get new predictions
		 */
		for(Agent agent : _lastPredictions.keySet()) {
			_lastPredictions.put(agent, agent.getActionDistribution());
		}
	}

	public PredictionByPartialMatching getCopy() {
		return new PredictionByPartialMatching(_g,_seed,_options);
	}

	public String toString() {
		return "Prediction By Partial Matching";
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new PredictionByPartialMatching(g,seed,options);
	}
}