package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;

import util.Array;

import brownstrategy.agent.NondeterministicAgent;

/**
 *
 * This is an abstract class for all of the previous sequence
 * oracles.
 * 
 * @author jberg
 *
 */

public abstract class AbstractPrevSequenceOracle extends NondeterministicAgent {

	protected int _numLastMoves = 1;
	protected ArrayList<Integer> player1Moves,player2Moves,player3Moves;
	private boolean SGT = false;
	private boolean LINSMOOTH = false;
	private double linSmooth = .075;

	public AbstractPrevSequenceOracle(Game g, long seed, String options) {
		super(g,seed,options);
		StringTokenizer st = new StringTokenizer(options, ",");
		if (st.hasMoreTokens()) {
			st.nextToken();
			if (st.hasMoreTokens()) {
				_numLastMoves = Integer.parseInt(st.nextToken());
				if (st.hasMoreTokens()) {
					SGT = Boolean.parseBoolean(st.nextToken());
				}
			}
		}
		player1Moves = new ArrayList<Integer>();
		player2Moves = new ArrayList<Integer>();
		player3Moves = new ArrayList<Integer>();
	}

	public abstract int[] getOppDist(int opp);

	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		int[] oppDist =  getOppDist(_playerIdx);
		if(SGT) {
			_cacheDist= SimpleGoodTuring.SGT(oppDist);
		}
		else if(LINSMOOTH) {
			_cacheDist = PlusNSmoothing.PlusNSmoothing(oppDist,linSmooth);
		}
		else {
			_cacheDist = normalize(oppDist);
		}
		return _cacheDist;
	}

	public void observeOutcome(int[] actions) {
		player1Moves.add(actions[0]);
		player2Moves.add(actions[1]);
		player3Moves.add(actions[2]);
		_day++;
	}
	
	@Override
	public void lookAhead(int[] actions) {
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		player1Moves.remove(player1Moves.size()-1);
		player2Moves.remove(player2Moves.size()-1);
		player3Moves.remove(player3Moves.size()-1);
		_day--;
	}

	/*
	 * This method should only take arrays of the same size!
	 */
	public boolean arrListEquals(List<Integer> arrList1, List<Integer> arrList2) {
		for(int i = 0; i < arrList1.size(); i++) {
			if (arrList1.get(i) != arrList2.get(i)) {
				return false;
			}
		}
		return true;
	}

}