package brownstrategy.oracles;

import game.Game;
import game.Strategy;

import java.util.Random;
import java.util.Stack;

import brownstrategy.agent.Agent;
import brownstrategy.agent.DeterministicAgent;

/**
 *
 * This oracle just returns what our opponents
 * did on the previous day as our prediction
 * for the next day
 * 
 * @author jberg
 *
 */

public class SameAsLastTimeOracle extends DeterministicAgent {

	int lastAction;
	private Stack<Integer> lastActionStack;
	Random rand;
	long _seed;

	public SameAsLastTimeOracle(Game g, long seed, String options) {
		super(g, seed, options);
		_seed = seed;
		rand = new Random(seed);
		lastAction = rand.nextInt(12); //Random on day 0.
		lastActionStack = new Stack<Integer>();
	}

	public int getAction() {
		return lastAction;
	}

	public void observeOutcome(int[] actions) {
		lastAction = actions[0];
	}
	
	public String toString() {
		return "SameAsLastOracle";
	}
	
	public Agent getCopy() {
		return new SameAsLastTimeOracle(_g,_seed,_options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new SameAsLastTimeOracle(g,seed,options);
	}

	@Override
	public void lookAhead(int[] actions) {
		lastActionStack.push(lastAction);
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		lastAction = lastActionStack.pop();
	}
	
}
