package brownstrategy.oracles;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

/**
 *
 * Simple Good-Turing Estimator
 * @author jberg
 *
 */

public class SimpleGoodTuring {


	/*
	 * This method should take in an array of length 12, with
	 * the number of times the sequence we are interested in
	 * ended with each value.  This is all we need to compute
	 * the SGT estimates.
	 */
	public static double[] SGT(int[] arr) {
		HashMap<Integer,ArrayList<Integer>> rmap = new HashMap<Integer, ArrayList<Integer>>();
		int tot0freq = 0;
		for(int i = 0; i < arr.length; i++) {
			if(arr[i] != 0) {
				ArrayList<Integer> tmp = rmap.get(arr[i]);
				if(tmp == null) {
					tmp = new ArrayList<Integer>();
					tmp.add(i);
				}
				else {
					tmp.add(i);
				}
				rmap.put(arr[i], tmp);
			}
			else {
				tot0freq++;
			}
		}
		
		/*
		 * If there are too few frequencies, default
		 * to plus 1 smoothing
		 */
		if(rmap.keySet().size() <= 1) {
			return PlusNSmoothing.PlusNSmoothing(arr,.25);
		}
		
		if(tot0freq == 0 || tot0freq == arr.length) {
			return normalize(arr);
		}
		

		ArrayList<Integer> r = new ArrayList<Integer>();

		for (int i : rmap.keySet()) {
			r.add(i);
		}

		Collections.sort(r);

		ArrayList<Double> logr = new ArrayList<Double>();
		ArrayList<Integer> n = new ArrayList<Integer>();
		for (int i : r) {
			logr.add(Math.log(i));
			n.add(rmap.get(i).size());
		}

		int N = 0;

		for(int i = 0; i < n.size(); i++) {
			N += r.get(i) * n.get(i);
		}

		double P0 = n.get(0)/((double) N);
		double P0per = P0/tot0freq;

		ArrayList<Double> Z = new ArrayList<Double>();
		ArrayList<Double> logZ = new ArrayList<Double>();		
		for(int j = 0; j < r.size(); j++) {
			int i,k;
			if(j == 0) {
				i = 0;
				k = r.get(j+1);
			}
			else if (j == r.size()-1) {
				i = r.get(j-1);
				k = 2*r.get(j) - i;
			}
			else {
				i = r.get(j-1);
				k = r.get(j+1);
			}
			double newZ = (2.0 * n.get(j)) / ((double) (k-i));
			Z.add(newZ);
			logZ.add(Math.log(newZ));
		}

		/*
		 * Find a,b for:
		 * logZ ~ a + b*logr
		 */
		double[] coeff = linReg(logr,logZ);
		double a = coeff[0];
		double b = coeff[1];
		
		ArrayList<Double> rstar = new ArrayList<Double>();
		boolean xflag = false;
		for(int i = 0; i < r.size(); i++) {
			int R = r.get(i);
			double y = (R+1) * (antilog(a,b,R+1)/antilog(a,b,R));

			if(xflag) {
				if((i == r.size()-1) || (R+1 != r.get(i+1))) {
					xflag = false;
				}
			}

			if(xflag) {
				double Niplus1 = n.get(i+1);
				double Ni = n.get(i);
				double x = (R+1) * (Niplus1/Ni);
				double xyTest = 1.96 * Math.sqrt((R+1)*(R+1) * (Niplus1/(Ni*Ni)) * (1 + (Niplus1/Ni)));
				if(Math.abs(x-y) > xyTest) {
					rstar.add(x);
				}
				else {
					xflag = false;
					rstar.add(y);
				}
			}
			else {
				rstar.add(y);
			}
		}
		
		double newN = 0;
		for(int i = 0; i < rstar.size(); i++) {
			newN += rstar.get(i) * n.get(i);
		}
		
		double[] newArr = new double[arr.length];
		for(int i = 0; i < r.size(); i++) {
			ArrayList<Integer> tmp = rmap.get(r.get(i));
			double prob = (1-P0)*(rstar.get(i)/newN);
			for(int j = 0; j < tmp.size(); j++) {
				newArr[tmp.get(j)] = prob;
			}
		}
		
		for(int i = 0; i < newArr.length; i++) {
			if(newArr[i] == 0) {
				newArr[i] = P0per;
			}
		}
		return newArr;
	}

	/*
	 * Code stolen from:
	 * 
	 * http://www.cs.princeton.edu/introcs/97data/LinearRegression.java.html
	 * 
	 */
	public static double[] linReg(ArrayList<Double> x, ArrayList<Double> y) { 
		int n = x.size();

		// first pass: read in data, compute xbar and ybar
		double sumx = 0.0, sumy = 0.0, sumx2 = 0.0;
		for(int i = 0; i < n; i++) {
			sumx  += x.get(i);
			sumx2 += x.get(i) * x.get(i);
			sumy  += y.get(i);
		}
		double xbar = sumx / n;
		double ybar = sumy / n;

		// second pass: compute summary statistics
		double xxbar = 0.0, yybar = 0.0, xybar = 0.0;
		for (int i = 0; i < n; i++) {
			xxbar += (x.get(i) - xbar) * (x.get(i) - xbar);
			yybar += (y.get(i) - ybar) * (y.get(i) - ybar);
			xybar += (x.get(i) - xbar) * (y.get(i) - ybar);
		}
		double beta1 = xybar / xxbar;
		double beta0 = ybar - beta1 * xbar;

		double[] coeff = new double[2];
		coeff[0] = beta0;
		coeff[1] = beta1;
		
		if(Double.isNaN(beta0) || Double.isNaN(beta1)) {
			System.out.println("shitfuck");
			System.out.println(x);
			System.out.println(y);
		}
		
//		// print results
//		System.out.println("y   = " + beta1 + " * x + " + beta0);
//
//		// analyze results
//		int df = n - 2;
//		double rss = 0.0;      // residual sum of squares
//		double ssr = 0.0;      // regression sum of squares
//		for (int i = 0; i < n; i++) {
//			double fit = beta1*x.get(i) + beta0;
//			rss += (fit - y.get(i)) * (fit - y.get(i));
//			ssr += (fit - ybar) * (fit - ybar);
//		}
//		double R2    = ssr / yybar;
//		double svar  = rss / df;
//		double svar1 = svar / xxbar;
//		double svar0 = svar/n + xbar*xbar*svar1;
//		System.out.println("R^2                 = " + R2);
//		System.out.println("std error of beta_1 = " + Math.sqrt(svar1));
//		System.out.println("std error of beta_0 = " + Math.sqrt(svar0));
//		svar0 = svar * sumx2 / (n * xxbar);
//		System.out.println("std error of beta_0 = " + Math.sqrt(svar0));
//
//		System.out.println("SSTO = " + yybar);
//		System.out.println("SSE  = " + rss);
//		System.out.println("SSR  = " + ssr);
		
		return coeff;
	}

	public static double antilog(double a, double b, int r) {
		return Math.exp(a + b*Math.log(r));
	}
	
	/**
	 * Normalizes a distribution so that it sums to 1.
	 * @param distribution
	 * @return
	 */
	public static double[] normalize(int[] distribution ) {
		double[] normDistribution = new double[distribution.length];
		double sum = 0;
		for (int i=0; i<distribution.length; i++) {
			sum += distribution[i];
		}

		/*
		 * added handling for sum is zero.  this shouldn't ever
		 * happen, but if it does it returns a unifrom distribution
		 * 
		 * -jberg
		 */
		if(sum == 0) {
			double uniform = 1.0/distribution.length;;
			for (int i=0; i<distribution.length; i++) {
				normDistribution[i] = uniform;
			}
		}
		else {
			for (int i=0; i<distribution.length; i++) {
				normDistribution[i] = distribution[i] / sum;
			}
		}
		return normDistribution;
	}

}