package strategy;

import game.Game;
import game.Strategy;
import game.Utility;
import java.util.Random;
import java.util.Arrays;
import lemonade.LemonadeUtility;
import strategy.Player;
import strategy.Agents;
import static java.lang.Math.*;

/* Author: Dustin Morrill
 * Date: September 7, 2010
 *
 * Agent description:
 *
 */

public class MyUtils {
    int num_players;
    int num_actions;
    Agents agents;
    Random generator;
    int mode;
    int num_rounds_in_curr_mode;
    int round;
    int modelling_threshold;

    public MyUtils (int np, int na, long random_seed) {
        num_players = np;
        num_actions = na;
        agents = new Agents(num_players);
        generator = new Random(random_seed + 94476);
        mode = 0;  // Mode zero is constant
        num_rounds_in_curr_mode = 0;
        modelling_threshold = 3;
        round = 0;
    }

// Player interface methods
    public Player getOpponentWithMostTotalUtil () {
        int total_rank[] = agents.getTotalRank();
        int indices[] = new int[total_rank.length];
        int index = -1;
        int j = 0;
        for (int i = 1; i < total_rank.length-1; i++) {
            //System.out.printf("constant_rank[%d]: %d, constant_rank[%d]: %d\n", i, constant_rank[i], i+1, constant_rank[i+1]);
            if (total_rank[i] < total_rank[i+1]) {
                indices[j] = i;
                j++;
            }
            else if (total_rank[i] == total_rank[i+1]) {
                indices[j] = i;
                j++;
                indices[j] = i+1;
                j++;
            }
            else {
                indices[j] = i+1;
                j++;
            }
        }
        if (j != 1) {
            index = indices[breakTie(indices)];
        }
        else {
            index = indices[0];
        }
        return getPlayer(index);
    }

    public Player getOpponentWithMostLatestUtil () {
        // If there is a tie in determining the agent with the maximum utility,
        // use the utility from the previous round to break the tie
        //System.out.printf("In breakConstantTie: indices[%d]: %d, indices[%d]: %d\n", 0, indices[0], 1, indices[1]);
        int latest_rank[] = agents.getLatestRank();
        int latest_indices[] = new int[latest_rank.length];
        int index = -1;
        int j = 0;
        for (int i = 1; i < latest_rank.length-1; i++) {
            //System.out.printf("latest_rank[%d]: %d, latest_rank[%d]: %d\n", indices[i], latest_rank[indices[i]], indices[i+1], latest_rank[indices[i+1]]);
            if (latest_rank[i] < latest_rank[i+1]) {
                latest_indices[j] = i;
                j++;
            }
            else if (latest_rank[i] == latest_rank[i+1]) {
                latest_indices[j] = i;
                j++;
                latest_indices[j] = i+1;
                j++;
            }
            else {
                latest_indices[j] = i+1;
                j++;
            }
            //System.out.printf("j: %d, l_i[0]: %d\n", j, latest_indices[0]);
        }
        // If it's still tied after that, flip a coin
        if (j != 1) {
            index = latest_indices[breakTie(latest_indices)];
            //System.out.printf("second tie broken: %d\n", index);
        }
        else {
            index = latest_indices[0];
            //System.out.printf("one index: %d\n", index);
        }
        //System.out.printf("Index returned: %d\n", index);
        return getPlayer(index);
    }

    public Player getMostConstantOpponent () {
        if (round < 2) {
            return getPlayer(generator.nextInt(1)+1);
        }
        int constant_labels[] = getConstantOpponentLabels();
        int index = -1;
        //System.out.printf("indices[%d]: %d, indices[%d]: %d\n", 0, indices[0], 1, indices[1]);
        //System.out.printf("j: %d\n", j);
        if (constant_labels.length != 1) {
            index = constant_labels[breakTie(constant_labels)];
            //System.out.printf("tie broken: %d\n", index);
        }
        else {
            index = constant_labels[0];
            //System.out.printf("one index: %d\n", index);
        }
        //System.out.printf("index returned: %d\n", index);
        return getPlayer(index);
    }

    public int[] getConstantOpponentLabels () {
        int constant_rank[] = agents.getConstantRank();
        int indices[] = new int[constant_rank.length-1];
        int j = 0;
        for (int i = 1; i < constant_rank.length-1; i++) {
            //System.out.printf("constant_rank[%d]: %d, constant_rank[%d]: %d\n", i, constant_rank[i], i+1, constant_rank[i+1]);
            if (constant_rank[i] < constant_rank[i+1]) {
                indices[j] = i;
                j++;
            }
            else if (constant_rank[i] == constant_rank[i+1]) {
                indices[j] = i;
                j++;
                indices[j] = i+1;
                j++;
            }
            else {
                indices[j] = i+1;
                j++;
            }
        }
        int constant_labels[] = new int[j];
        System.arraycopy(indices, 0, constant_labels, 0, j);
        return constant_labels;
    }

    public Player[] getConstantOpponents () {
        int labels[] = getConstantOpponentLabels();
        Player c_opponents[] = new Player[labels.length];
        for (int i = 0; i < labels.length; i++) {
            c_opponents[i] = getPlayer(labels[i]);
        }
        return c_opponents;
    }

    public Player getPlayer (int index) {
        return agents.getPlayerArray()[index];
    }

    public Player getMyPlayer () {
        return getPlayer(0);
    }

    public Player[] getRandomOpponents() {
        // Search through the list of players (other than yourself) to find one
        // that appears constant
        int indices[] = new int[num_players-1];
        int j = 0;
        for (int i = 1; i < num_players; i++) {
            if (getPlayer(i).isRandom()) {
                indices[j] = i;
                j++;
            }
        }
        Player random_players[] = new Player[j];
        for (int i = 0; i < random_players.length; i++) {
            random_players[i] = getPlayer(indices[i]);
        }
        return random_players;
    }

    public boolean bothDefensive () {
        boolean both_defensive = true;
        for (int i = 1; i < num_players; i++) {
            if (!getPlayer(i).isDefensive()) {
                both_defensive = false;
            }
        }
        return both_defensive;
    }

    public boolean isPlayer (Player p1, Player p2) {
        return p1.getLabel() == p2.getLabel();
    }

    public boolean isMyPlayer (Player p) {
        return isPlayer(p, getMyPlayer());
    }

// Target methods
    public void identifyTarget () {
        Player constant_players[] = getConstantOpponents();
        if (constant_players.length > 0) {
            int index = 0;
            if (constant_players.length > 1) {
                // If more than one player appears to be playing constant,
                // target the leading player
                if (constant_players[0].getTotalUtil() < constant_players[1].getTotalUtil()) {
                    index = 1;
                }
                // If they are equal, pick the one you have already been
                // targeting
                else if (constant_players[0].getTotalUtil() == constant_players[1].getTotalUtil()) {
                    if (targetLocked()) {
                        if (isPlayer(getTarget(), constant_players[1])) {
                            index = 1;
                        }
                    }
                    // Otherwise flip a coin
                    else {
                        index = generator.nextInt(constant_players.length);
                    }
                }
            }
            setTarget(constant_players[index]);
        }
        else {
            unsetTarget();
        }
    }

    public void setTarget (Player target) {
        agents.setTargetLabel(target.getLabel());
    }

    public void unsetTarget () {
        agents.setTargetLabel(-1);
    }

    public boolean targetLocked () {
        return agents.getTargetLabel() > 0;
    }

    public Player getTarget () {
        return getPlayer(agents.getTargetLabel());
    }

    public Player getOtherOpponent () {
        return getPlayer(agents.getTargetLabel() == 1 ? 2 : 1);
    }

// Mode methods
    public void setDefenseMode (Player target) {
        agents.setTargetLabel(target.getLabel());
        if (mode != 1) {
            mode = 1;
            num_rounds_in_curr_mode = 0;
        }
        int constant_labels[] = getConstantOpponentLabels();
        if (constant_labels.length != 1) {
            getMyPlayer().setPos(oppositePos(getOtherOpponent().getLatestPos()));
        }
        else {
            getMyPlayer().setPos(oppositePos(target.getLatestPos()));
        }
    }

    public boolean inDefenseMode () {
        return mode == 1;
    }

    public void setAttackMode(Player target) {
        agents.setTargetLabel(target.getLabel());
        if (mode != 2) {
            mode = 2;
            num_rounds_in_curr_mode = 0;
        }
        getMyPlayer().setPos(oppositePos1AboutPos2(getOtherOpponent().getLatestPos(), getTarget().getLatestPos()));
    }

    public boolean inAttackMode () {
        return mode == 2;
    }

    public void setDesperationMode (Player target) {
        agents.setTargetLabel(target.getLabel());
        if (mode != 3) {
            mode = 3;
            num_rounds_in_curr_mode = 0;
        }
        int num_rounds_in_phaseI = modelling_threshold * 2;
        // Offer a sandwich attack
        if (num_rounds_in_curr_mode < num_rounds_in_phaseI) {
            if (num_rounds_in_curr_mode == modelling_threshold) {
                agents.setTargetLabel(getOtherOpponent().getLabel());
            }
            getMyPlayer().setPos(oppositePos1AboutPos2(
                    getOtherOpponent().getLatestPos(),
                    getTarget().getLatestPos()));
        }
        // Sit on a player if neither wants to attack the other
        else {
            int num_rounds_in_phaseII = num_rounds_in_curr_mode - num_rounds_in_phaseI;
            if ((double) num_rounds_in_phaseII % (double) modelling_threshold == 0) {
                int num_thresholds_in_phaseII = num_rounds_in_phaseII / modelling_threshold;
                // If this is the first time in phaseII or a number equal to
                // 3*2^x where x is some integer, change your target
                // This ensures that an exponentially increasing amount of time
                // is used try and force an opponent to move
                if (num_thresholds_in_phaseII == 0 ||
                   (log((double)num_thresholds_in_phaseII)/log((double)2) ==
                   floor(log((double)num_thresholds_in_phaseII)/log((double)2)))
                   ) {
                    if (num_thresholds_in_phaseII == 1) {
                        // If you've offered an attack to an opponent, attacked
                        // that opponent, and sat on that opponent, it should
                        // be modelled as defensive and it shows no desire to
                        // attack the opponent
                        target.setDefensive();
                    }
                    agents.setTargetLabel(getOtherOpponent().getLabel());
                }
            }
            getMyPlayer().setPos(getTarget().getLatestPos());
        }
    }

    public boolean inDesperationMode () {
        return mode == 3;
    }

    public void unsetMode () {
        mode = 0;
    }

// Ranking methods
    public int getAverageLatestUtil () {
        int average = 0;
        int latest_util[] = getMyPlayer().getLatestUtilArray(2);
        for (int i = 0; i < latest_util.length; i++) {
            average += latest_util[i];
        }
        average = average / latest_util.length;
        return average;
    }

    public int breakTie (int indices[]) {
        return generator.nextInt(indices.length);
    }

    public void updateRankings () {
        // Set the total and latest ranking
        // Find the player with the maximum total utility
        int len = agents.getPlayerArray().length;

        // Arrays to hold the final rankings
        int total_ranking[] = new int[len];
        int latest_ranking[] = new int[len];
        int constant_ranking[] = new int[len];

        // Arrays to hold raw scores for each player
        int total_scores[] = new int[len];
        int latest_scores[] = new int[len];
        int constant_scores[] = new int[len];

        // Get player scores
        for (int i = 0; i < len; i++) {
            Player curr_player = getPlayer(i);
            total_scores[i] = curr_player.getTotalUtil();
            latest_scores[i] = curr_player.getLatestUtil();
            constant_scores[i] = curr_player.getConstantScore();
        }
        // Sort the three arrays
        Arrays.sort(total_scores);
        Arrays.sort(latest_scores);
        Arrays.sort(constant_scores);
        // Reverse the order
        for (int i = 0; i < len/2; i++) {
            int temp = total_scores[i];
            total_scores[i] = total_scores[total_scores.length-1-i];
            total_scores[total_scores.length-1-i] = temp;
            temp = latest_scores[i];
            latest_scores[i] = latest_scores[latest_scores.length-1-i];
            latest_scores[latest_scores.length-1-i] = temp;
            temp = constant_scores[i];
            constant_scores[i] = constant_scores[constant_scores.length-1-i];
            constant_scores[constant_scores.length-1-i] = temp;
        }
        // Figure out which player has which score
        for (int i = 0; i < len; i++) {
            Player curr_player = getPlayer(i);
            total_ranking[curr_player.getLabel()] = len+1;
            latest_ranking[curr_player.getLabel()] = len+1;
            constant_ranking[curr_player.getLabel()] = len+1;
            for (int j = 0; j < len; j++) {
                int rank = j+1;
//System.out.printf("j: %d, ts: %d, cts: %d, ls: %d, cls: %d, cs: %d, ccs: %d\n",
//j, total_scores[j], curr_player.getTotalUtil(), latest_scores[j], curr_player.getLatestUtil(), constant_scores[j], curr_player.getConstantScore());
                if (total_scores[j] == curr_player.getTotalUtil() && total_ranking[curr_player.getLabel()] > rank) {
                    total_ranking[curr_player.getLabel()] = rank;
  //                  System.out.printf("tr[%d]: %d\n", curr_player.getLabel(), total_ranking[curr_player.getLabel()]);
                }
                if (latest_scores[j] == curr_player.getLatestUtil() && latest_ranking[curr_player.getLabel()] > rank) {
                    latest_ranking[curr_player.getLabel()] = rank;
    //                System.out.printf("lr[%d]: %d\n", curr_player.getLabel(), latest_ranking[curr_player.getLabel()]);
                }
                if (constant_scores[j] == curr_player.getConstantScore() && constant_ranking[curr_player.getLabel()] > rank) {
                    constant_ranking[curr_player.getLabel()] = rank;
      //              System.out.printf("cr[%d]: %d\n", curr_player.getLabel(), constant_ranking[curr_player.getLabel()]);
                }
            }
        }
        agents.setTotalRank(total_ranking);
        agents.setLatestRank(latest_ranking);
        agents.setConstantRank(constant_ranking);
        //System.out.printf("tr after: %d\n", agents.getConstantRank()[0]);
    }

// Situation methods
    public boolean opponentsAreTogether () {
        boolean are_together = true;
        for (int i = 1; i < num_players-1; i++) {
            int diff = abs(getPlayer(i).getLatestPos() - getPlayer(i+1).getLatestPos());
            if (diff > 1) {
                are_together = false;
            }
        }
        return are_together;
    }

    public boolean opponentsAreAcross () {
        boolean are_across = true;
        for (int i = 1; i < num_players-1; i++) {
            int diff = abs(getPlayer(i).getLatestPos() - getPlayer(i+1).getLatestPos());
            if (diff != num_actions/2) {
                are_across = false;
            }
        }
        return are_across;
    }

    public boolean positionsAreTogether (int pos1, int pos2) {
        boolean are_together = true;
        for (int i = 1; i < num_players-1; i++) {
            int diff = abs(pos2 - pos1);
            if (diff > 1) {
                are_together = false;
            }
        }
        return are_together;
    }

    public boolean positionsAreAcross (int pos1, int pos2) {
        boolean are_across = true;
        for (int i = 1; i < num_players-1; i++) {
            int diff = abs(pos2 - pos1);
            if (diff != num_actions/2) {
                are_across = false;
            }
        }
        return are_across;
    }

// Position methods
    public int oppositePos1AboutPos2(int pos1, int pos2) {
        int my_new_pos;
        int diff = pos2 - pos1;
        if (diff == 0) {
            my_new_pos = pos2 == 0 ? pos2 + 1 : pos2 - 1;
        }
        // If the difference is greater than the number of actions divided by
        // two, the new position must be less than pos2
        else if(diff > num_actions / 2) {
            my_new_pos = pos2 - 1;
        }
        // If it's less than the negative of the number of actions divided by
        // two, the new position must be greater than pos2
        else if (diff < -(num_actions / 2)){
            my_new_pos = pos2 + 1;
        }
        else if (pos1 < pos2) {
            my_new_pos = pos2 + 1 < num_actions ? pos2 + 1 : 0;
        }
        else {
            my_new_pos = pos2 - 1 >= 0 ? pos2 - 1 : num_actions - 1;
        }
        return my_new_pos;
    }

    public int oppositePos (int pos) {
        return pos < 6 ? pos + 6 : pos - 6;
    }

    public boolean sharingOneSpace () {
        int my_pos = getMyPlayer().getLatestPos();
        boolean s_o_s = false;
        for (int i = 1; i < num_players; i++) {
            if (my_pos == getPlayer(i).getLatestPos()) {
                s_o_s = true;
            }
        }
        return s_o_s;
    }

// Interface methods
    public int getModellingThreshold () {
        return modelling_threshold;
    }

    public void updateRound () {
        round++;
        num_rounds_in_curr_mode++;
    }

    public int getRound () {
        return round;
    }

    public int getNumActions () {
        return num_actions;
    }

    public int getNumPlayers () {
        return num_players;
    }

    public int getRandomInt (int range) {
        return generator.nextInt(range);
    }

    public Agents getAgents () {
        return agents;
    }

    public int getNumRoundsInCurrMode () {
        return num_rounds_in_curr_mode;
    }
}