package edu.cmu.psy.reitter.meta;

import game.Game;
import game.Strategy;
import game.Utility;
import java.util.Random;
import lemonade.LemonadeUtility;


import static java.lang.Math.*;


/**
 * A strategy that chooses a random action on the first round, and then
 * plays that action thereafter until utility falls below 7 for a specified number of trials (randomly chosen).
 * When that happens a new action is chosen as well as a new number of trails to wait for utility to fall
 * @author Unmesh Kurup
 * based on work by @author maz
 *
 */
public class ConstantStrategy_PlusRandom implements Strategy{
  /**
   * The action the player plays
   */
  int constantAction;
  
  int numActions; //used to select the max of the number of times of stay put. Max number should ideally be based on number of trials
  int maxTimesLowUtility; //how long to wait before jumping location
  int numTimesLowUtility;  //how many times has utility been low
  Utility utility;
  Random r;
  
  
  /**
   * Construct a constant strategy. The game is
   * used to get the number of actions, and the
   * seed is used to seed a pseudorandom number 
   * generator, which then is queried for the action 
   * to take.
   * 
   * @param g
   * @param seed
   */
  public ConstantStrategy_PlusRandom(Game g, long seed, String options) {
    numActions = g.getNumActions();
    r = new Random(seed);
    constantAction = r.nextInt(numActions);
    //lowUtilityForXTries = false;
    utility = g.getUtility();
    maxTimesLowUtility = 1+r.nextInt(numActions); //randomly select the number of times to stay put.
    numTimesLowUtility = 0;
  }
  



  @Override
  public int getAction() {
    return constantAction;
  }

  @Override
  public void observeOutcome(int[] actions) {
	  if (utility.getUtility(actions)[0]<7) {

	      // sandwich attack?
	      if (abs(actions[1]-actions[0])==1 && abs(actions[2]-actions[0])==1)
		  {if (r.nextFloat()>=0.5)
			  constantAction = (max(actions[1],actions[2])+1)%12;
		      else
			  constantAction = (min(actions[1],actions[2])+11)%12;
		  }
	      else
		  numTimesLowUtility++;
	  }
	  else {
		  numTimesLowUtility = 0;
	  }
	  
	  if (numTimesLowUtility>maxTimesLowUtility) {
		  constantAction = r.nextInt(numActions);
		  maxTimesLowUtility = r.nextInt(numActions);
		  numTimesLowUtility = 0;
	  }
  }
  
  
  
}

