package ca.ualberta.cs.shamooshak;

import game.Game;
import game.Strategy;

public class Shamooshak
  implements Strategy
{
  public static final int ROUNDS = 100;
  int actionCounter;
  int[][] action;
  double[][][] prediction;
  double[][] weight;
  Predictor[] predictor;
  Game g;
  int[] score;
  int[] lastScore;
  public static double alfa = 0.75D;
  public static double beta = 0.3D;

  public Shamooshak(Game g, long seed, String options) {
    this.actionCounter = 0;
    this.action = new int[100][3];
    this.score = new int[3];
    this.lastScore = new int[3];
    for (int i = 0; i < 3; i++) {
      this.score[i] = 0;
      this.lastScore[i] = 0;
    }
    this.g = g;

    this.predictor = new Predictor[6];
    this.predictor[0] = new ConstPredictor();
    this.predictor[1] = new AgainstThemPredictor();
    this.predictor[2] = new AgainstUsPredictor();
    this.predictor[3] = new BestResponsePredictor();
    this.predictor[4] = new MiddlePredictor();
    this.predictor[5] = new SequencePredictor();

    this.prediction = new double[this.predictor.length][2][12];
    this.weight = new double[this.predictor.length][2];
    for (int i = 0; i < this.predictor.length; i++)
      for (int j = 0; j < 2; j++)
        this.weight[i][j] = 0.33D;
  }

  void add_action(int[] p)
  {
    for (int i = 0; i < 3; i++) {
      this.score[i] += this.g.getUtility().getUtility(p)[i];
    }

    for (int j = 0; j < 3; j++)
      this.action[this.actionCounter][j] = p[j];
    this.actionCounter += 1;

    for (int i = 0; i < this.predictor.length; i++)
      for (int j = 0; j < 2; j++)
        this.weight[i][j] = (alfa * this.weight[i][j] + (1.0D - alfa) * (2.0D * this.prediction[i][j][p[j]] + this.prediction[i][j][((p[j] + 1) % 12)] + this.prediction[i][j][((p[j] + 11) % 12)]));
  }

  int next_move()
  {
    double[][] ans = new double[2][12];
    for (int i = 0; i < this.predictor.length; i++) {
      this.predictor[i].predict(this.prediction[i], this.action, this.actionCounter);
      for (int player = 0; player < 2; player++)
        for (int place = 0; place < 12; place++)
          ans[player][place] += this.prediction[i][player][place] * this.weight[i][player];
    }
    for (int player = 0; player < 2; player++) {
      double sum = 0.0D;
      for (int place = 0; place < 12; place++)
        sum += ans[player][place];
      for (int place = 0; place < 12; place++)
        ans[player][place] /= sum;
    }
    double max = 0.0D;
    int maxi = -1;
    for (int place = 0; place < 12; place++) {
      if (max < ans[0][place] + ans[1][place]) {
        max = ans[0][place] + ans[1][place];
        maxi = place;
      }
    }
    if (this.actionCounter < 2)
      return 6;
    return (maxi + 6) % 12;
  }

  public int getAction()
  {
    return next_move();
  }

  public void observeOutcome(int[] arg0)
  {
    int[] action = new int[3];
    action[0] = arg0[1];
    action[1] = arg0[2];
    action[2] = arg0[0];
    add_action(action);
  }
}