/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package edu.uwaterloo.strategy;

import com.yahoo.lsg.lifelong.GameInformation;
import com.yahoo.lsg.lifelong.LifelongStrategy;
import game.Game;
import game.Utility;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Arrays;
import java.util.Random;
import lemonade.GeneralizedLemonadeUtility;
/**
 *
 * @author taha
 */
public class CombinedStrategy implements LifelongStrategy {
  protected Utility utility;
  public int numActions;
  public Random r;

    
    
    private final int STATE_NORMAL = 0;
    private final int STATE_PUNISH = 1;    
    private final int STATE_MAXIMIZE = 2;
    private final int STATE_CONSTANT = 3;
    private final int STATE_COLLUDE = 4; 
    
    private final int AREA_CONST = 4;
    private final int PUNISH_THRESH = 8;
    private final int COLLUDE_THRESH = 20;
    private final float RELAX_CONST = 1.0f;
    private int myAction;
//    private PrintStream out;
//    private PrintStream result;
    
    private int[] utilSum; //maintained during game
    private int[] history; //maintained during life - game outcomes
    private int boardAvg;
    private int turn;
    
    private int state;
    
    //Punish state variables
    private int toPunish;
    private int punishCounter;
    private int punished;

    //Constant state variables
    private int[] changeHistory;
    private int[] lastMoves;
    
    PrintStream list;

    public CombinedStrategy(GameInformation info, long seed, String options) throws IOException{
        this.r = new Random(seed);

//      out = new PrintStream(new FileOutputStream("GameLog.txt",true));
//      out.println("- New Lifetime ---------------------------------");
//      result = new PrintStream(new FileOutputStream("Result.txt",true));
//      System.setOut(result);
      
      utilSum = new int[3];
      history = new int[]{-1,0,0};
      changeHistory = new int[]{0,0,0};
      lastMoves = new int[]{0,0,0};
    }
    
    @Override
    public void newGame(Game g, int iterations) {
        this.numActions = g.getNumActions();
        this.utility = g.getUtility();


        //save outcome of previous game
        if (utilSum[0] >= utilSum[1] && utilSum[0] >= utilSum[2]) history[0]++;
        else if (utilSum[0] <= utilSum[1] && utilSum[0] <= utilSum[2]) history[2]++;
        else history[1]++;

//        if (boardAvg != 0) {
//            if (utilSum[0] >= boardAvg*100) out.print("WIN \t");
//            else out.print("LOSS \t");
//            out.println("MARGIN: " + String.valueOf(((float)utilSum[0]/100) / (float)boardAvg));            
//        }

        //out.println(Array.arrayToNakedString(history) + "\t\t" + Array.arrayToNakedString(utilSum));
        //out.println("-----------------NEW GAME----------------");
        
        //Collusion - Bowling
        this.U = (GeneralizedLemonadeUtility)g.getUtility();
        initBestActions();
        constantAction = bestActions[r.nextInt(bestActions.length)];
        scoreAction = r.nextInt(48 * 2);
        scoreResponse = 0;
        colludeCount = 0;
        
        //Reset parameters..
        for (int i = 0; i < 3; i++) {utilSum[i] = 0; changeHistory[i] = 0; lastMoves[i] = 0;}
        boardAvg = 0;
        turn = 0;
        state = STATE_COLLUDE;
        toPunish = -1;
        punishCounter = 0;
        punished = 0;

        //Do some calculations
        myAction = constantAction;
        boardAvg = calculateAvg();
        
        
    }

    @Override
    public void observeOutcome(int[] actions) {
//        out.println("----");
//        out.println("Actions: " + Array.arrayToNakedString(actions));
//        out.println("Utilities: " + Array.arrayToNakedString(this.utility.getUtility(actions)));
//        out.println("Utility Sums: " + Array.arrayToNakedString(utilSum));
//        out.println("State: " + String.valueOf(state));        

        for (int i = 0; i < 3; i++) {
            if (actions[i]!=lastMoves[i]) changeHistory[i]++;
            lastMoves[i] = actions[i];
        }
        
        turn++;        
        
        int[] utils = this.utility.getUtility(actions);
        for (int i = 0; i < 3; i++)
            utilSum[i] += utils[i];
        
        switch (state) {
            case STATE_NORMAL:
                doNormal(actions,utils);
                break;
            case STATE_PUNISH:
                doPunish(actions,utils);
                break;
            case STATE_MAXIMIZE:
                doMaximize(actions,utils);
                break;
            case STATE_CONSTANT:
                doConstant(actions,utils);
                break;
            case STATE_COLLUDE:
                doCollude(actions,utils);
        }

    }
    
    public int[] getPossibleUtilities(int[] actions){
        int[] possibleUtilities=new int[numActions];
        for(int i=0;i<possibleUtilities.length;i++){
            actions[0]=i;
            possibleUtilities[i]=utility.getUtility(actions)[0];
        }
        return possibleUtilities;
    }

    @Override
    public int getAction() {
        return myAction;
    }
    
    
    //Custom functions    
    
    private int bestAction() {
        int[] actions = new int[3];
        int[] utils = new int[3];
        int[] areaValues = new int[12];

        for (int i = 0; i < 12; i++) {
            actions[0] = i;
            actions[1] = i-AREA_CONST;
            if (actions[1] < 0) actions[1] += 12;
            actions[2] = i+AREA_CONST;           
            if (actions[2] > 11) actions[2] -= 12;

            this.utility.getUtility(actions,utils);
            areaValues[i] = utils[0];
        }
        
        int max = 0;
        int maxVal = areaValues[0];
        for (int i = 1; i < 12; i++) {
            if (areaValues[i] > maxVal) {
                maxVal = areaValues[i];
                max = i;
            }
        }
        
        return max;
    }
    
    private int calculateAvg() {
        int avg = this.utility.getUtility(new int[]{0,0,0})[0];
//        out.println(String.format("Board Average: %d",avg));
        return avg;
    }
    
    private int findNearest(int[] actions) {
        int diff1 = Math.abs(actions[1] - actions[0]);
        int diff2 = Math.abs(actions[2] - actions[0]);
        if (diff1 > 6) diff1 = 12 - diff1;
        if (diff2 > 6) diff2 = 12 - diff2;
        return (diff1<diff2)?1:2;
    }
    
    private int[] findMax(int[] acts) {
        int[] actions = Arrays.copyOf(acts, 3);
        int max = 0;
        int maxIndex = -1;
        for (int i = 0; i < 12; i++) { //find max
            actions[0] = i;
            int util = this.utility.getUtility(actions)[0];
            if (util > max) {
                max = util;
                maxIndex = i;
            }
        }
        return new int[]{max,maxIndex};
    }
    
    private int findConstant(int[] acts) {
        
        
        int[] actions = Arrays.copyOf(acts, 3);        
        int max = 0;
        int maxIndex = -1;
        for (int i = 0; i < 3; i++) {
            if (changeHistory[i] >= max) {
                max = changeHistory[i];
                maxIndex = i;
            }
        }
        
        int toCollaborate = actions[maxIndex];
        int myMove = toCollaborate + 6;
        if (myMove >= 12) myMove -= 12;
        return myMove;
//        if (maxIndex == 1) actions[2] = toCollaborate;
//        else actions[1] = toCollaborate;
//        return findMax(actions)[1];
    }
    
    private int punish(int nearest, int[] acts) {
        int[] actions = Arrays.copyOf(acts, 3);
        int min = 999;
        int minIndex = -1;
        for (int i = 0; i < 12; i++) {
            actions[0] = i;
            int[] utils = this.utility.getUtility(actions);
            if (utils[nearest] < min) {
                min = utils[nearest];
                minIndex = i;
            }
        }
        return minIndex;
    }
    
    private void doNormal(int[] actions,int[] utils) {
        if (utils[0] >= utils[1] && utils[0] >= utils[2]) {
            //Do nothing
        }
        else if (utils[0] <= utils[1] && utils[0] <= utils[2]) {
            //Do smart stuff
            //Is there any other good spot?
            int[] max = findMax(actions);
            if (max[0] > RELAX_CONST*boardAvg) { // yes - go to spot
                myAction = max[1];
            }            
            else {
                toPunish = findNearest(actions);
                myAction = punish(toPunish,actions);
                state = STATE_PUNISH;                
            }
            
        }
        else {
            //Check if im performing okay and stay
        }
        
    }
    
    private void doPunish(int[] actions, int[] utils) {
            //Is there any other good spot?
            int[] max = findMax(actions);
            if (max[0] > RELAX_CONST*boardAvg) { // yes - go to spot
                myAction = max[1];
                punishCounter = 0;
                punished = 0;
                state = STATE_NORMAL;
            }            
            else { //no - keep punishing
                if (punishCounter >= PUNISH_THRESH) {
                    punishCounter = 0;
                    if (punished == 0) {
                        if (toPunish == 1) toPunish = 2;
                        else toPunish = 1;
                        myAction = punish(toPunish,actions);
                        punished++;
                    }
                    else {
                        myAction = findMax(actions)[1];
                        state = STATE_MAXIMIZE;                        
                    }
                }
                //myAction = punish(toPunish,actions);
                punishCounter++;
            }
    }

    private void doMaximize(int[] actions, int[] utils) {
        int[] max = findMax(actions);
        myAction = max[1];
    }

    private void doConstant(int[] actions, int[] utils) {

    }    

    
    //Collude
    
    int constantAction;
    int[] bestActions;

    int[] quality;
    int[] response;

    int scoreAction, scoreResponse;
    GeneralizedLemonadeUtility U;
    int totalu;

    int colludeCount;
    
    private void doCollude(int[] actions, int[] utils) {
        colludeCount++;
        
        if (colludeCount >=COLLUDE_THRESH) {
            if (utilSum[0] < boardAvg*colludeCount) {
                myAction = bestAction();
                state = STATE_NORMAL;
                return;
            }
            else colludeCount = 0;
        }
        
        scoreAction += this.U.getUtility(new int[] {constantAction, actions[1], actions[2]})[0];
        scoreResponse += this.U.getUtility(new int[] { response[constantAction], actions[1], actions[2]})[0];

        if (scoreAction <= scoreResponse) myAction = response[constantAction];
        else myAction = constantAction;    
    }    
    
    
    protected int collusionFairness(int a, int b) {
	int[] u = U.getUtility(new int[]{a, b});
	return u[0] - u[1];
    }

    protected int collusionSuccess(int a, int b) {
	int min = totalu;
	for (int c=0; c<U.getIslandSize(); c++) {
	    int[] u = U.getUtility(new int[]{a, b, c});
	    if (u[0] + u[1] < min) min = u[0] + u[1];
	}

	return min;
    }

    protected int getQuality(int action){
	int bestv = 0;

	for (int b=0; b < U.getIslandSize(); b++) {
	    int s = collusionSuccess(action, b);
	    int f = collusionFairness(action, b);
	    int v;

	    if (3 * Math.abs(f) > totalu) f *= (-1);

	    v = 1000000 * s - 1000 * Math.abs(f) - f;

	    if (v > bestv) { 
		bestv = v;
		response[action] = b;
	    }
	}

	return bestv;
    }
    
  private void initBestActions(){
    quality = new int[U.getIslandSize()];
    response = new int[U.getIslandSize()];

    totalu = 0;
    for(int i=0; i<U.getIslandSize(); i++) totalu += U.getNumConsumers(i);

    for(int i=0;i<quality.length;i++){
      quality[i]=getQuality(i);
    }
    int maxDensity = quality[0];
    int maxDensityCount = 1;
    for(int i=1;i<quality.length;i++){
      
      if (maxDensity<quality[i]){
        maxDensity=quality[i];
        maxDensityCount=1;
      } else if (maxDensity==quality[i]){
        maxDensityCount++;
      }
    }
    bestActions = new int[maxDensityCount];
    int bestActionIndex = 0;
    for(int i=0;i<quality.length;i++){
      if (quality[i]==maxDensity){
        bestActions[bestActionIndex]=i;
        bestActionIndex++;
      }
    }
  }

    
}
