package analysis;

public class FitnessFunction {
  ThreePlayerGame game;
  double minimumValue;
  double maximumValue;
  
  public FitnessFunction(ThreePlayerGame game){
    this.game = game;
    
    minimumValue=game.data[0][0][0][0];
    maximumValue=game.data[0][0][0][0];
    for(int i=0;i<game.data.length;i++){
      for(int j=0;j<game.data.length;j++){
        for(int k=0;k<game.data.length;k++){
          double current = game.data[i][j][k][0];
          if (current<minimumValue){
            minimumValue=current;
          }
          if (current>maximumValue){
            maximumValue=current;
          }
        }
      }
    }
  }
  
  public double[] getSolution(){
    double[] distribution = new double[game.data.length];
    double prob = 1.0/distribution.length;
    for(int i=0;i<distribution.length;i++){
      distribution[i]=prob;
    }
    update(distribution,100000);
    return distribution;
  }
  
  /**
   * This fitness function is very conservative, in that the fitness of each action is between
   * 99 and 100.
   * @param action
   * @param distribution
   * @return
   */
  public double getFitness(int action, double[] distribution){
    double util = game.getUtility(action,distribution);
    return (util-minimumValue)/(maximumValue-minimumValue)+99.0;
  }
  
  

  public double[] getUtilVector(double[] distribution){
    double[] utilVector=new double[distribution.length];
    for(int action=0;action<utilVector.length;action++){
      utilVector[action] = game.getUtility(action,distribution);
    }
    return utilVector;
  }
  
  public double getSolutionRegret(double[] distribution){
    
    double[] utilVector=getUtilVector(distribution);
    double solutionUtil = 0;
    for(int action=0;action<utilVector.length;action++){
      solutionUtil+=(utilVector[action]*distribution[action]);
    }

    double maxRegret = utilVector[0]-solutionUtil;
    for(int action=0;action<utilVector.length;action++){
      double currentRegret = utilVector[action]-solutionUtil;
      if (currentRegret>maxRegret){
        maxRegret = currentRegret;
      }
    }
    return maxRegret;
  }
  
  
  public void update(double[] distribution, int iterations){
    
    double[] fitness = new double[distribution.length];
    for(int i=0;i<iterations;i++){
    for(int action=0;action<fitness.length;action++){
      fitness[action]=getFitness(action,distribution);
    }
    double total = 0;

    for(int action=0;action<fitness.length;action++){
      distribution[action]=distribution[action]*fitness[action];
      total+=distribution[action];
    }
    for(int action=0;action<fitness.length;action++){
      distribution[action]/=total;
    }
    }
  }
  
}
