package analysis;
import java.util.Random;


public class TruncatedUniformDistribution implements UtilityDistribution {


  public double getProbabilityEqualTo(int k, int maxValue){
    // Pr[X=k|Success]=Pr[X=k AND Success]/Pr[Success]
    // Pr[X=k|Success]=Pr[X=k AND X+Y<=M](1/M+1)/Pr[Success]
    // Pr[X=k|Success]=Pr[X=k AND Y<=M-k](1/M+1)/Pr[Success]
    // Pr[X=k|Success]=Pr[X=k](M+1-k)/(M+1)^2/Pr[Success]
    // Pr[X=k|Success]=(M+1-k)/((M+1)^3 Pr[Success])

    return ((double)(maxValue+1-k))/(getProbabilitySuccess(maxValue)*(maxValue+1)*(maxValue+1)*(maxValue+1));
  }
  
  public double getProbabilityLessThanOrEqualTo(int k, int maxValue){
    // Pr[X=k|Success]=(M+1-k)/((M+1)^3 Pr[Success])
    // Pr[X<=k|Success]=sum_{j=0}^k(M+1-j)/((M+1)^3 Pr[Success])
    // Pr[X<=k|Success]=((k+1)(M+1)-(k)(k+1)/2)/((M+1)^3 Pr[Success])
    // Pr[X<=k|Success]=(k+1)(2M+2-k)/(2(M+1)^3 Pr[Success])
    // Pr[X<=k|Success]=(k+1)(2M+2-k)/(2(M+1)^3 Pr[Success])
    // Pr[Success]=(maxValue + 2)/((maxValue + 1)*(maxValue+1)*2)
    // Pr[X<=k|Success]=(k+1)(2M+2-k)/((M+2)(M+1))
    int numerator = (k+1)*(2*maxValue+2-k);
    int denominator = (maxValue+1)*(maxValue+2);
    return ((double)numerator)/((double)denominator);
  }
  
  public double getProbabilitySuccess(int maxValue){
    int numerator = (maxValue + 2);
    int denominator = (maxValue + 1)*(maxValue+1)*2;
    return ((double)numerator)/((double)denominator);
  }
  
  public int dartToValue(double dart, int maxValue){
    for(int value=0;value<=maxValue;value++){
      double prob = getProbabilityLessThanOrEqualTo(value,maxValue);
      if (dart<prob){
        return value;
      }
    }
    return maxValue;
  }
  
  @Override
  public void generateUtilities(int[] values, int maxValue, Random r) {
    double dart = r.nextDouble();
    values[0]=dartToValue(dart,maxValue);
    
    values[1]=r.nextInt(maxValue+1-values[0]);
    values[2]=maxValue-(values[0]+values[1]);
  }

}
