package com.yahoo.lsg.correlatedresults;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import results.TripleResults;

public class CorrelatedDifference {
  public static RandomVariable compare(List<DetailedResults> results, String playerA, String playerB){
    List<DetailedResults> shortList = new ArrayList<DetailedResults>();
    for(DetailedResults result:results){
      if (result.getIndex(playerA)!=-1 || result.getIndex(playerB)!=-1){
        shortList.add(result);
      }
    }
    
    int length = shortList.iterator().next().detailedResults.length;
    
    
    RandomVariable difference = new RandomVariable();
    for(int i=0;i<length;i++){
      long totalForA=0;
      long totalForB=0;
      for(DetailedResults result:shortList){
        int indexA = result.getIndex(playerA);
        int indexB = result.getIndex(playerB);
        if (indexA!=-1){
          totalForA += result.detailedResults[i][indexA];
        }
        if (indexB!=-1){
          totalForB += result.detailedResults[i][indexB];
        }
      }
      difference.addSample(totalForA-totalForB);
    }
    return difference;
  }

  
  
  
  public static Hashtable<String,RandomVariable> getDifferences(Hashtable<String,DetailedRandomVariable> totals){
    Hashtable<String,RandomVariable> differences = new Hashtable<String,RandomVariable>();
    for(String minuend:totals.keySet()){
      for(String subtrahend:totals.keySet()){
        DetailedRandomVariable difference = totals.get(minuend).difference(totals.get(subtrahend));
        RandomVariable compressedDifference = new RandomVariable(difference);
        differences.put(minuend+"-"+subtrahend,compressedDifference);
      }
    }
    return differences;
  }
  
  
  public static Hashtable<String,DetailedRandomVariable> getTotalsAgain(List<DetailedResults> results){
    Hashtable<String,DetailedRandomVariable> totals = new Hashtable<String,DetailedRandomVariable>();
    for(DetailedResults result:results){
      for(Map.Entry<String, DetailedRandomVariable> entry:result.getDetailedRandomVariables().entrySet()){
        String key = entry.getKey();
        DetailedRandomVariable currentVariable = entry.getValue();
        if (totals.containsKey(key)){
          DetailedRandomVariable oldVar = totals.get(key);
          DetailedRandomVariable nextVar = oldVar.sum(currentVariable);
          totals.put(key, nextVar);
        } else {
          totals.put(key, currentVariable);
        }
      }
    }
    return totals;
  }
  
  
  public static Map<String,RandomVariable> compress(Map<String,? extends RandomVariable> variables){
    Hashtable<String,RandomVariable> results = new Hashtable<String,RandomVariable>();
    for(Map.Entry<String, ? extends RandomVariable> entry:variables.entrySet()){
      results.put(entry.getKey(),new RandomVariable(entry.getValue()));
    }
    return results;
  }
  
  
  public static class ReverseOrder<V> implements Comparator<V>{
    Comparator<V> comparator;
    public ReverseOrder(Comparator<V> comparator){
      this.comparator = comparator;
    }
    @Override
    public int compare(V o1, V o2) {
      return comparator.compare(o2,o1);
    }
  }
  
  public static class ValueComparator<V> implements Comparator<Map.Entry<?, V>>{

    Comparator<V> comparator;
    public ValueComparator(Comparator<V> comparator){
      this.comparator = comparator;
    }
    @Override
    public int compare(Entry<?, V> o1, Entry<?, V> o2) {
      return comparator.compare(o1.getValue(),o2.getValue());
    }
    
  }
  
  public static class RandomVariableComparator implements Comparator<RandomVariable>{

    @Override
    public int compare(RandomVariable o1, RandomVariable o2) {
      double mean1=o1.getMean();
      double mean2=o2.getMean();
      double diff = mean1-mean2;
      return(int)(Math.signum(diff));
    }
    
  }
  public static Hashtable<String,DetailedRandomVariable> getTotals(List<DetailedResults> results){
    String[] names = TripleResults.getNames(results);
    Hashtable<String,DetailedRandomVariable> finalResult = new Hashtable<String,DetailedRandomVariable>();
    for(String name:names){
      finalResult.put(name,new DetailedRandomVariable());
    }

    int length = results.iterator().next().detailedResults.length;
    
    
    for(int i=0;i<length;i++){
      LongSummation<String> summation = new LongSummation<String>();
      
      for(DetailedResults result:results){
        for(int j=0;j<result.playerNames.length;j++){
          summation.add(result.playerNames[j],result.detailedResults[i][j]);
        }
      }
      for(Map.Entry<String, Long> entry:summation.entrySet()){
        finalResult.get(entry.getKey()).addSample(entry.getValue());
      }
    }
    return finalResult;
  }
  
  public static List<DetailedResults> getTranslatedResults(File resultsFile, Hashtable<String,String> mapping) throws IOException{
    List<DetailedResults> results = DetailedResults.getDetailedResults(resultsFile);
    TripleResults.translate(results, mapping);
    return results;
  }
  
  static String[] places = new String[]{"Zeroeth","First","Second","Third","Fourth","Fifth","Sixth","Seventh","Eighth","Ninth","Tenth"};
  static String[] suffixes = new String[]{"th","st","nd","rd"};
  public static String getPlace(int place){
    if (place<1){
      throw new RuntimeException("There is no such place as "+place);
    }
    if (place<places.length){
      return places[place];
    }
    int lastDigit = place % 10;
    if (lastDigit<suffixes.length){
      return ""+place+suffixes[lastDigit];
    } else {
      return "th";
    }
  }
  
  
  public static String getRoundedString(double value, int digits){
    StringWriter buffer = new StringWriter();
    PrintWriter writer = new PrintWriter(buffer);
    writer.format("%."+digits+"f", value);
    return buffer.toString();
  }
  public static void main(String[] args) throws IOException{
    File mappingFile = new File(args[0]);
    List<File> inputFiles = new ArrayList<File>();
    for(int i=1;i<args.length;i++){
      inputFiles.add(new File(args[i]));
    }
    Hashtable<String,String> mapping = new Hashtable<String,String>();
    TripleResults.loadHashtable(mappingFile, mapping);
    
    RandomVariableSummation<String> summation = new RandomVariableSummation<String>();
    RandomVariableSummation<String> differenceSum = new RandomVariableSummation<String>();
    for(File resultsFile:inputFiles){
      System.err.println("Getting results from "+resultsFile.getName());
      List<DetailedResults> results = getTranslatedResults(resultsFile,mapping);
      System.err.println("Getting totals from "+resultsFile.getName());
      Hashtable<String,DetailedRandomVariable> totals = getTotals(results);
      results = null;
      System.err.println("Getting differences from "+resultsFile.getName());
      
      Hashtable<String,RandomVariable> differences = getDifferences(totals);
      System.err.println("Compressing totals from "+resultsFile.getName());
      
      Map<String,RandomVariable> compression = compress(totals);
      /*for(Map.Entry<String, RandomVariable> entry:compression.entrySet()){
        String key = entry.getKey();
        RandomVariable variable = entry.getValue();
        System.out.println(key+"="+variable.getMean()+"+/-"+variable.getStandardErrorOfTheMean());
      }*/
      summation.addAll(compression);
      differenceSum.addAll(differences);
      /*for(Map.Entry<String, RandomVariable> entry:differences.entrySet()){
        String key = entry.getKey();
        RandomVariable variable = entry.getValue();
        System.out.println(key+"="+variable.getMean()+"+/-"+variable.getStandardErrorOfTheMean());
      }*/
    }
    Comparator<Map.Entry<?, RandomVariable>> comparator = new ReverseOrder<Map.Entry<?, RandomVariable>>(new ValueComparator<RandomVariable>(new RandomVariableComparator()));
    ArrayList<Map.Entry<String,RandomVariable>> entries = new ArrayList<Map.Entry<String,RandomVariable>>(summation.entrySet());
    Collections.sort(entries,comparator);
    double scale = 1.0/(100.0*7*6*3);
    
    
    for(int index=0;index<entries.size();index++){
      Map.Entry<String, RandomVariable> entry = entries.get(index);
      String team = entry.getKey();
      RandomVariable variable = entry.getValue();
      double scaledUtility = (variable.getMean()*scale);
      double standardErrorOfTheMean = (variable.getStandardErrorOfTheMean()*scale);
      String place = getPlace(index+1);
      
      System.out.println("<TR><TD>"+place+"</TD><TD>"+team+"</TD><TD>"+getRoundedString(scaledUtility,3)+"+/-"+getRoundedString(standardErrorOfTheMean,3)+"</TD></TR>");
      //System.out.println(team+"="+scaledUtility+"+/-"+standardErrorOfTheMean);
    }
    
    for(int i=0;i+1<entries.size();i++){
      String lower = entries.get(i+1).getKey();
      String higher = entries.get(i).getKey();
      String differenceKey = higher+"-"+lower;
      RandomVariable differenceValue = differenceSum.get(differenceKey);
      double standardErrorOfTheMean = differenceValue.getStandardErrorOfTheMean()*scale;
      double difference = differenceValue.getMean()*scale;
      
      double zScore = differenceValue.getZScore();
      //double cdf = differenceValue.getCDF();
      //double pvalue = (1.0-cdf)*2.0;
      
      //differenceValue.getMean()/differenceValue.getStandardErrorOfTheMean();
      System.out.println("<TR><TD>"+differenceKey+"</TD><TD>"+getRoundedString(difference,3)+"+/-"+getRoundedString(standardErrorOfTheMean,3)+"</TD><TD>z-score:"+getRoundedString(zScore,3)+"</TD></TR>");
    }
    
    
    //String[] names = CalculateMeans.getRankedNames(results);
    //Hashtable<String,Double> means = CalculateMeans.getMeans(results);
    /* for(int i=0;i<names.length;i++){
      System.out.println(""+(i+1)+":"+names[i]+" ("+means.get(names[i])+")");
      if (i<names.length-1){
        RandomVariable r = compare(results,names[i],names[i+1]);
        System.out.println("Difference:"+r.getMean()+"+/-"+r.getStandardErrorOfTheMean());
      }
    } */
  }
}
