package com.yahoo.lsg.correlatedresults;


public class DetailedRandomVariable extends RandomVariable{
  long[] samples;
  
  public DetailedRandomVariable(){
    samples = new long[10];
  }
  
  public void addSamples(DetailedRandomVariable variable){
    for(int i=0;i<variable.count;i++){
      addSample(variable.samples[i]);
    }
  }
  
  public void addSample(long sample){
    if (count==Integer.MAX_VALUE){
      throw new RuntimeException("Cannot store more than "+Integer.MAX_VALUE+ " examples");
    }
    if (samples.length<count){
      throw new RuntimeException("Count is larger than the samples collected");
    }
    if (samples.length==count){
      long[] temp = new long[((int)count+1)*2];
      System.arraycopy(samples, 0, temp, 0, samples.length);
      samples=temp;
    }
    samples[(int)count]=sample;
    total+=sample;
    totalOfSquares +=(sample*sample);
    count++;
  }
  
  
  
  
  public double getMean(){
    return ((double)total)/count;
  }
  

  public double getStandardErrorOfTheMean(){
    double mean = ((double)total)/count;
    double meanOfSquares = ((double)totalOfSquares)/count;
    double biasedVariance = meanOfSquares - (mean * mean);
    double unbiasedVariance = (biasedVariance * count)/(count-1);
    double oneExampleStandardDeviation = Math.sqrt(unbiasedVariance);
    double standardErrorOfTheMean = oneExampleStandardDeviation/Math.sqrt(count);
    return standardErrorOfTheMean;
  }
  
  public DetailedRandomVariable opposite(){
    DetailedRandomVariable result = new DetailedRandomVariable();
    for(int i=0;i<count;i++){
      result.addSample(-samples[i]);
    }
    return result;
  }
  
  
  public DetailedRandomVariable sum(DetailedRandomVariable otherAddend){
    DetailedRandomVariable result = new DetailedRandomVariable();
    if (count!=otherAddend.count){
      throw new RuntimeException("Random variables do not have the same number of samples");
    }
    for(int i=0;i<count;i++){
      result.addSample(samples[i]+otherAddend.samples[i]);
    }
    return result;
  }
  
  /**
   * Returns a random variable which is 
   * this-subtrahend
   * @param subtrahend the thing to subtract
   * @return this-subtrahend
   * @throws a runtime exception if this.count!=subtrahend.count
   */
  public DetailedRandomVariable difference(DetailedRandomVariable subtrahend){
    return sum(subtrahend.opposite());
  }
}
