package com.yahoo.lsg.correlatedresults;


public class RandomVariable {
  long total;
  long totalOfSquares;
  long count;
  public void addSample(long sample){
    total+=sample;
    totalOfSquares +=(sample*sample);
    count++;
  }
  
  public RandomVariable(){
    total=0;
    totalOfSquares=0;
    count=0;
  }
  public RandomVariable(RandomVariable variable){
    this.total = variable.total;
    this.count = variable.count;
    this.totalOfSquares = variable.totalOfSquares;
  }
  
  public void addSamples(RandomVariable variable){
    this.total+=variable.total;
    this.count+=variable.count;
    this.totalOfSquares +=variable.totalOfSquares;
  }
  
  public double getMean(){
    return ((double)total)/count;
  }
  
  public double getZScore(){
    return getMean()/getStandardErrorOfTheMean();
  }
  
  static final int TERMS=90;
  public static double erf(double z){
    double term=2.0*z/Math.sqrt(Math.PI);
    double tally=term;
    for(int n=1;n<TERMS;n++){
      term*=(-(z*z)/n);
      tally+=(term/(2*n+1));
    }
    return tally;
  }
  
  /**
   * Basically, past six standard deviations, the numbers are highly unreliable.
   * For example, erf(5.0) returns a number greater than 1.0.
   * @return
   */
  public double getCDF(){
    double z = getZScore()/Math.sqrt(2.0);
    if (z>4.5){
      return 1.0;
    }
    return 0.5*(1.0+erf(z));
  }

  /**
   * The standard error of the mean. This can be used to construct a confidence
   * interval.
   * @return
   */
  public double getStandardErrorOfTheMean(){
    double mean = ((double)total)/count;
    double meanOfSquares = ((double)totalOfSquares)/count;
    double biasedVariance = meanOfSquares - (mean * mean);
    double unbiasedVariance = (biasedVariance * count)/(count-1);
    double oneExampleStandardDeviation = Math.sqrt(unbiasedVariance);
    double standardErrorOfTheMean = oneExampleStandardDeviation/Math.sqrt(count);
    return standardErrorOfTheMean;
  }
  
}
