package com.yahoo.lsg.lifelong;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public class PlayerIdentity{
  LifelongStrategy strategy;
  final private String playerClass;
  final private String playerOption;
  final private Constructor<LifelongStrategy> constructor;
  
  
  public String getPlayerClass(){
    return playerClass;
  }
  
  public PlayerIdentity(String playerClass, String playerOption) throws SecurityException, ClassNotFoundException, NoSuchMethodException{
    this.playerClass = playerClass;
    this.playerOption = playerOption;
    constructor = getConstructor(playerClass);
    strategy = null;
  }
  
  
  
  public void initialize(GameInformation info, long seed) throws IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException{
    strategy = constructor.newInstance(info, seed,playerOption);
  }
  
  private static Constructor<LifelongStrategy> getConstructor(String className) throws ClassNotFoundException, SecurityException, NoSuchMethodException{
    Class<?> original = Class.forName(className);
    if (!LifelongStrategy.class.isAssignableFrom(original)){
      throw new ClassCastException(className+" does not implement LifelongStrategy or Strategy");
    }
    
    @SuppressWarnings("unchecked")
    Class<LifelongStrategy> correctType = (Class<LifelongStrategy>)original;
    
    Constructor<LifelongStrategy> constructor = correctType.getConstructor(GameInformation.class,long.class,String.class);
    return constructor;
  }


  public void endLifetime() {
    strategy = null;
  }
  
  



}
