package com.yahoo.lsg.lifelong;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public class PlayerRecord implements Comparable<PlayerRecord>{
  private double utility;
  final DoubleRandomVariable lifetimeUtility;
  private long countRounds;
  LifelongStrategy strategy;
  final private String playerClass;
  final private String playerOption;
  final private Constructor<LifelongStrategy> constructor;
  
  
  public PlayerRecord(String playerClass, String playerOption) throws SecurityException, ClassNotFoundException, NoSuchMethodException{
    this.playerClass = playerClass;
    this.playerOption = playerOption;
    constructor = getConstructor(playerClass);
    strategy = null;
    utility = 0;
    lifetimeUtility = new DoubleRandomVariable();
  }
  
  
  
  public void initialize(GameInformation info, long seed) throws IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException{
    strategy = constructor.newInstance(info, seed,playerOption);
  }
  
  private static Constructor<LifelongStrategy> getConstructor(String className) throws ClassNotFoundException, SecurityException, NoSuchMethodException{
    Class<?> original = Class.forName(className);
    if (!LifelongStrategy.class.isAssignableFrom(original)){
      throw new ClassCastException(className+" does not implement LifelongStrategy or Strategy");
    }
    
    @SuppressWarnings("unchecked")
    Class<LifelongStrategy> correctType = (Class<LifelongStrategy>)original;
    
    Constructor<LifelongStrategy> constructor = correctType.getConstructor(GameInformation.class,long.class,String.class);
    return constructor;
  }

  public void incrementUtility(double utilityDelta){
    utility+=utilityDelta;
    countRounds++;
  }

  public void endLifetime() {
    
    final double utilityPerRound;
    if (countRounds==0){
      utilityPerRound = 0;
    } else {
      utilityPerRound = (utility/countRounds);
    }
    
    lifetimeUtility.add(utilityPerRound);
    utility = 0;
    countRounds = 0;
    strategy = null;
  }
  
  
  public void display(){
    System.out.println("Player:"+this.playerClass+" Utility:"+lifetimeUtility.getMean()+" StandardDeviation:"+lifetimeUtility.getStandardErrorOfTheMean()); 
  }



  @Override
  public int compareTo(PlayerRecord o) {
    Double thisValue = lifetimeUtility.getMean();
    Double otherValue = o.lifetimeUtility.getMean();
    return thisValue.compareTo(otherValue);
  }
}
