package com.yahoo.lsg.lifelong.examples;

import game.Game;
import game.Utility;

import java.util.Random;

import com.yahoo.lsg.lifelong.LifelongStrategy;


/**
 * An abstract class for strategies which depend only upon
 * the utility of the action taken and the utility the player 
 * would have gotten had it taken another strategy.
 * 
 * @see ExternalRegretMatchingStrategy
 * 
 * This class also provides a random number generator. It is unclear
 * if this is the best place for it.
 * @author maz
 *
 */
public abstract class UtilityBasedLifelongStrategy implements LifelongStrategy{
  Utility utility;
  public int numActions;
  public Random r;
  
  public void newGame(Game g, int iterations){
    this.numActions = g.getNumActions();
    this.utility = g.getUtility();
  }
  
  /**
   * Note that a lifelong strategy must implement
   * a constructor with (GameInformation, long, String)
   * @param seed the random seed
   */
  public UtilityBasedLifelongStrategy(long seed) {
    
    this.r = new Random(seed);
  }

  public int[] getPossibleUtilities(int[] actions){
    int[] possibleUtilities=new int[numActions];
    for(int i=0;i<possibleUtilities.length;i++){
      actions[0]=i;
      possibleUtilities[i]=utility.getUtility(actions)[0];
    }
    return possibleUtilities;
  }
  
  public void observeOutcome(int[] actions){
    int[] possibleUtilities = getPossibleUtilities(actions);
    int currentAction = actions[0];
    observeUtilities(possibleUtilities,currentAction);
  }

  /**
   * When observeOutcome() is called, the utilities are calculated
   * and passed as an array.
   * @param possibleUtilities the utilities of all possible actions.
   * @param currentAction 
   */
  public abstract void observeUtilities(int[] possibleUtilities, int currentAction);
}
