package com.yahoo.lsg.lifelong.observer;

import java.util.IdentityHashMap;

import game.Game;

import com.yahoo.lsg.lifelong.GameInformation;
import com.yahoo.lsg.lifelong.PlayerIdentity;


/**
 * This class is declared abstract because it does not have the constructor
 * required for LifelongObserver. See UtilityObserver for an example.
 * @author maz
 *
 */
public abstract class PlayerFocusedObserver implements LifelongObserver {
  final GameInformation info;
  final IdentityHashMap<PlayerIdentity,PlayerObserver> map;  
  final PlayerObserver []observers;

  
  PlayerIdentity[] allIdentities;
  Game currentGame;
  PlayerObserver []currentObservers;
  int[][] rotatedActions;
  
  
  public PlayerFocusedObserver(PlayerObserver[] observers, GameInformation info, PlayerIdentity []identities){
    this.info = info;
    this.allIdentities = identities;
    this.observers = observers;
    map = new IdentityHashMap<PlayerIdentity,PlayerObserver>();
    for(int i=0;i<observers.length;i++){
      map.put(allIdentities[i], observers[i]);
    }
  }
  
  
  @Override
  public void newGame(Game g, PlayerIdentity... identities) {
    currentObservers = new PlayerObserver[identities.length];
    for(int i=0;i<currentObservers.length;i++){
      currentObservers[i]=map.get(identities[i]);
    }
    
    for(PlayerObserver observer:currentObservers){
      observer.newGame(g);
    }
    rotatedActions = new int[currentObservers.length][currentObservers.length];
    // TODO Auto-generated method stub

  }

  
  
  
  
  @Override
  public void observeOutcome(int[] actions) {
    for(int j=0;j<rotatedActions.length;j++){
      for(int k=0;k<rotatedActions.length;k++){
        rotatedActions[j][k]=actions[k];
      }
      rotatedActions[j][j]=actions[0];
      rotatedActions[j][0]=actions[j];
    }
    for(int i=0;i<currentObservers.length;i++){
      currentObservers[i].observeOutcome(rotatedActions[i]);
    }

    
  }

  public void endLifetime(){
    for(PlayerObserver observer:observers){
      observer.endLifetime();
    }
  }


  @Override
  public void display() {
    
    for(PlayerObserver observer:observers){
      observer.display();
    }
  }

}
