package com.yahoo.lsg.lifelong.observer;

import game.Game;


import com.yahoo.lsg.lifelong.DoubleRandomVariable;
import com.yahoo.lsg.lifelong.PlayerIdentity;

public class PlayerUtilityObserver implements Comparable<PlayerUtilityObserver>,PlayerObserver{
  final PlayerIdentity identity;
  
  private double utility;
  final DoubleRandomVariable lifetimeUtility;
  private long countRounds;
  private Game currentGame;
  private double averageUtility;
  
  

  public static double getAverageUtility(Game g){
    final int[] actions = new int[g.getNumPlayers()];
    final int[] utilExample = g.getUtility().getUtility(actions);
    int totalUtil = 0;
    for(int util:utilExample){
      totalUtil+=util;
    }
    return ((double)totalUtil)/((double)(actions.length));
  }
  
  
  public PlayerUtilityObserver(PlayerIdentity identity){
    this.identity = identity;
    utility = 0;
    lifetimeUtility = new DoubleRandomVariable();
    countRounds = 0;
    currentGame = null;
  }
  
  
  
  
  
  public void incrementUtility(double utilityDelta){
    utility+=utilityDelta;
    countRounds++;
  }

  public void endLifetime() {
    final double utilityPerRound;
    if (countRounds==0){
      utilityPerRound = 0;
    } else {
      utilityPerRound = (utility/countRounds);
    }
    
    lifetimeUtility.add(utilityPerRound);
    utility = 0;
    countRounds = 0;
  }
  
  
  public void display(){
    System.out.println("Player:"+this.identity.getPlayerClass()+" Utility:"+lifetimeUtility.getMean()+" StandardDeviation:"+lifetimeUtility.getStandardErrorOfTheMean()); 
  }



  @Override
  public int compareTo(PlayerUtilityObserver o) {
    Double thisValue = lifetimeUtility.getMean();
    Double otherValue = o.lifetimeUtility.getMean();
    return thisValue.compareTo(otherValue);
  }



  @Override
  public void newGame(Game g) {
    currentGame = g;
    averageUtility = getAverageUtility(currentGame);
    
  }



  @Override
  public void observeOutcome(int[] actions) {
    int[] util = currentGame.getUtility().getUtility(actions);
    incrementUtility(((double)util[0])-averageUtility);
  }
}
