package com.yahoo.lsg.stats.generalized;



import game.Game;
import game.GameGenerator;
import game.Utility;
import stats.SeriesObserver;

public class CollusionCount implements SeriesObserver {
  
  Game current;
  boolean currentHasNoCollusions;
  int numCollusions;
  int impossibleToCollude;
  int numRounds;
  int[][] detailCollusions;
  public CollusionCount(GameGenerator generator){
    current = null;
    numCollusions = 0;
    impossibleToCollude = 0;
    numRounds = 0;
    detailCollusions=new int[3][3];
  }
  
  
  public String toString(){
    return "{\"numCollusions\":"+numCollusions+",\"numRounds\":"+numRounds+",\"impossibleToCollude\":"+impossibleToCollude+"}";
  }
  
  public String toString(String[] playerNames){
    String playerString = "[\""+playerNames[0]+"\", \""+playerNames[1]+"\", \""+playerNames[2]+"\"]";
    return "{\"numCollusions\":"+numCollusions+",\"numRounds\":"+numRounds+",\"impossibleToCollude\":"+impossibleToCollude+",\"playerNames\":"+playerString+"}";
  }
  
  @Override
  public void display(String[] playerNames) {
    System.out.println(toString(playerNames));
  }

  /**
   * In this, we are guaranteeing that player 0 and
   * player 1 get at least as much as player 2, regardless of where
   * player 2 plays.
   * @param a
   * @param b
   * @return
   */
  public boolean isCollusion(int a, int b){
    int numActions = current.getNumActions(2);
    Utility util = current.getUtility();
    for(int c=0;c<numActions;c++){
      int[] profile = util.getUtility(new int[]{a,b,c});
      if (profile[0]<profile[2] || profile[1]<profile[2]){
        return false;
      }
    }
    return true;
  }
  
  
  @Override
  public void finishGame() {
    // TODO Auto-generated method stub
  }

  @Override
  public void finishSeries() {
    // TODO Auto-generated method stub

  }

  public boolean possibleToCollude(){
    int numActions = current.getNumActions();
    for(int i=0;i<numActions;i++){
      for(int j=i;j<numActions;j++){
        if (isCollusion(i,j)){
          return true;
        }
      }
    }
    return false;
  }
  
  @Override
  public void startGame(Game g) {
    current = g;
    if (!current.getUtility().isSymmetric()){
      throw new RuntimeException("Designed for symmetric games");
    }
    this.currentHasNoCollusions = !possibleToCollude();
  }

  /**
   * This function has side effects.
   * @param actions
   * @return
   */
  public boolean hasCollusion(int[] actions){
    boolean someCollusion=false;
    for(int i=0;i<3;i++){
      for(int j=i+1;j<3;j++){
        if (isCollusion(actions[i],actions[j])){
          detailCollusions[i][j]++;
          someCollusion=true;
        }   
      }
    }
    return someCollusion;
  }

  @Override
  public void observeOutcome(int[] actions) {
    if (hasCollusion(actions)){
      numCollusions++;
    }
    if (currentHasNoCollusions){
      impossibleToCollude++;
    }
    numRounds++;
  }

}
