package com.yahoo.lsg.stats.generalized;

import java.util.Arrays;

import game.Game;
import game.GameGenerator;
import game.Utility;
import stats.SeriesObserver;

public class StableWeakCollusionCount implements SeriesObserver{

  int countSame;
  int numRounds;
  int stableCollusions;
  int stableStrongCollusions;
  int[] lastActions;
  Game current;
  public StableWeakCollusionCount(GameGenerator generator){
    lastActions = null;
    countSame = 0;
  }
  
  
  public String toString(){
    return "{\"countSame\":"+countSame+",\"numRounds\":"+numRounds+",\"stableCollusions\":"+stableCollusions+",\"stableStrongCollusions\":"+stableStrongCollusions+"}";
  }
  
  public String toString(String[] playerNames){
    String playerString = "[\""+playerNames[0]+"\", \""+playerNames[1]+"\", \""+playerNames[2]+"\"]";
    return "{\"countSame\":"+countSame+",\"numRounds\":"+numRounds+",\"stableCollusions\":"+stableCollusions+",\"stableStrongCollusions\":"+stableStrongCollusions+",\"playerNames\":"+playerString+"}";
  }
  
  
  /**
   * In this, we are guaranteeing that player 0 +
   * player 1 >= 2 * player 2, regardless of where
   * player 2 plays.
   * @param a
   * @param b
   * @return
   */
  public boolean isWeakCollusion(int a, int b){
    int numActions = current.getNumActions(2);
    Utility util = current.getUtility();
    for(int c=0;c<numActions;c++){
      int[] profile = util.getUtility(new int[]{a,b,c});
      if (profile[0]+profile[1]<2*profile[2]){
        return false;
      }
    }
    return true;
  }

  public boolean isStableWeakCollusion(int[] actions, int index0,int index1){
    if (lastActions==null){
      return false;
    }
    if (!(lastActions[index0]==actions[index0])){
      return false;
    }
    if (!(lastActions[index1]==actions[index1])){
      return false;
    }
    return isWeakCollusion(actions[index0],actions[index1]);
  }

  /**
   * In this, we are guaranteeing that player 0 +
   * player 1 >= 2 * player 2, regardless of where
   * player 2 plays.
   * @param a
   * @param b
   * @return
   */
  public boolean isStrongCollusion(int a, int b){
    int numActions = current.getNumActions(2);
    Utility util = current.getUtility();
    for(int c=0;c<numActions;c++){
      int[] profile = util.getUtility(new int[]{a,b,c});
      if (profile[0]<profile[2] || profile[1]<profile[2]){
        return false;
      }
    }
    return true;
  }

  public boolean isStableStrongCollusion(int[] actions, int index0,int index1){
    if (lastActions==null){
      return false;
    }
    if (!(lastActions[index0]==actions[index0])){
      return false;
    }
    if (!(lastActions[index1]==actions[index1])){
      return false;
    }
    return isStrongCollusion(actions[index0],actions[index1]);
  }

  @Override
  public void display(String[] playerNames) {
    System.out.println(toString(playerNames));
  }

  @Override
  public void finishGame() {
    // TODO Auto-generated method stub
    
  }

  @Override
  public void finishSeries() {
    // TODO Auto-generated method stub
    
  }

  @Override
  public void startGame(Game g) {
    lastActions = null;
    current = g;
    // TODO Auto-generated method stub
    
  }

  public boolean closeEnough(int[] actions){
    if (lastActions==null){
      return false;
    }
    int numEqual = 0;
    for(int i=0;i<actions.length;i++){
      if (lastActions[i]==actions[i]){
        numEqual++;
      }
    }
    return (numEqual>=2);
  }
  
  public boolean isStableCollusion(int[] actions){
    if (isStableWeakCollusion(actions,0,1)){
      return true;
    }
    
    if (isStableWeakCollusion(actions,0,2)){
      return true;
    }
    
    return isStableWeakCollusion(actions,1,2);

  }

  public boolean isStableStrongCollusion(int[] actions){
    if (isStableStrongCollusion(actions,0,1)){
      return true;
    }
    
    if (isStableStrongCollusion(actions,0,2)){
      return true;
    }
    
    return isStableStrongCollusion(actions,1,2);

  }

  @Override
  public void observeOutcome(int[] actions) {
    assert(actions.length==3);
    if (isStableCollusion(actions)){
      stableCollusions++;
    }
    if (isStableStrongCollusion(actions)){
      stableStrongCollusions++;
    }
    
    
    if (closeEnough(actions)){
      countSame++;
    }
    lastActions = Arrays.copyOf(actions,actions.length);
    numRounds++;
  }

}
