package com.yahoo.lsg.sundisplay;

import java.util.Vector;

/**
 * An encapsulation for the model data. Synchronizes getting and setting of the model.
 * 
 * It is also possible to have a "write-lock" on the model.
 * @author maz
 *
 */
public class SunModel {
  private SunModelData data;
  
  private boolean lock;
  private Vector<SunModelListener> listeners;
  
  
  public SunModel(int numPlayers){
    data = new SunModelData(numPlayers);
    listeners = new Vector<SunModelListener>();
  }
  
  public void addListener(SunModelListener listener){
    listeners.add(listener);
  }
  
  
  public synchronized void setModel(SunModelData data){
    this.data.set(data);
    for(SunModelListener listener:listeners){
      listener.handleChange(this);
    }
  }
  
  public synchronized void getModel(SunModelData output){
    output.set(data);
  }
  
  public synchronized boolean attemptToLock(){
    if (lock){
      return false;
    }
    lock = true;
    return true;
  }
  
  public synchronized void releaseLock(){
    lock = false;
  }
  
  
  public void lockAndGetModel(SunModelData output) throws InterruptedException{
    while(!attemptToLock()){
      Thread.sleep(2000);
    }
    getModel(output);
  }
  
  public void setModelAndUnlock(SunModelData input){
    setModel(input);
    releaseLock();
  }
  
}
