package com.yahoo.lsg.sundisplay;


/**
 * A model for the view to display.
 * Presently, a number of "pieces" or "players"
 * can be visible or invisible. If they are visible, they
 * are located at multiple places.
 * 
 * TODO Make this implement the MVC pattern completely. In particular,
 * wrap this class with a "true model" and make this the model data.
 * @author maz
 *
 */
public class SunModelData {
  public int []positions;
  public int []lastActions;
  private boolean[] visible;
  private boolean[] faded;
  public int[] scores;
  public String currentGameIndex;
  
  private double timeOfDay;
  
  public boolean utilitiesVisible;
  public String message;

  
  public boolean isVisible(int player){
    return visible[player];
  }
  
  
  public double getTimeOfDay(){
    return timeOfDay;
  }
  
  public void setTimeOfDay(double timeOfDay){
    this.timeOfDay = timeOfDay;
  }
  
  public void setFadedAll(boolean fade){
    for(int i=0;i<faded.length;i++){
      faded[i]=fade;
    }
  }
  public void setFaded(int player, boolean fade){
    faded[player]=fade;
  }
  public boolean isFaded(int player){
    return faded[player];
  }
  
  
  
  public boolean allVisible(){
    for(int i=0;i<visible.length;i++){
      if (!visible[i]){
        return false;
      }
    }
    return true;
  }
  
  
  public SunModelData(int numPieces){
    positions = new int[numPieces];
    lastActions = new int[numPieces];
    visible = new boolean[numPieces];
    faded = new boolean[numPieces];
    scores = new int[numPieces];
    utilitiesVisible = false;
    message = null;
  }
  /**
   * Get the player index of the visible piece in a position.
   * If no visible player is there, return -1.
   * @param position
   * @return the player index of the visible piece in a position, or if
   * no visible player is there, return -1
   */
  public int getVisiblePiece(int position){
    for(int i=0;i<positions.length;i++){
      if (visible[i] && positions[i]==position){
        return i;
      }
    }
    return -1;
  }
  
  public void setPiece(int piece, int position){
    positions[piece]=position;
    visible[piece]=true;
  }

  public void setBeginGame(){
    setVisibleAll(false);
    setPiece(0, 0);
    setFaded(0,false);
    for(int i=0;i<scores.length;i++){
      scores[i]=0;
    }
    currentGameIndex = "";
    utilitiesVisible=false;
  }
  

  public void setVisibleAll(boolean isVisible) {
    for(int i=0;i<visible.length;i++){
      visible[i]=isVisible;
    }
  }


  public void set(SunModelData other) {
    System.arraycopy(other.positions,0,this.positions,0,this.positions.length);
    System.arraycopy(other.lastActions,0,this.lastActions,0,this.lastActions.length);
    System.arraycopy(other.visible,0,this.visible,0,this.visible.length);
    System.arraycopy(other.faded,0,this.faded,0,this.faded.length);
    System.arraycopy(other.scores,0,this.scores,0,this.scores.length);
    
    this.timeOfDay = other.timeOfDay;
    this.currentGameIndex = other.currentGameIndex;
    this.utilitiesVisible = other.utilitiesVisible;
    
    this.message=other.message;

  }


  public void setVisible(int i, boolean b) {
    visible[i]=b;
    
  }


  public void setPieceOld(int i, int oldPosition) {
    this.lastActions[i]=oldPosition;
    
  }
}
