package game;


import util.Array;

/**
 * A class giving the structure of a single shot game.
 * Designed to be unmodifiable, so that can be given to players.
 * NOTE: utility class for game must also be unmodifiable.
 * @author maz
 *
 */
public class Game {
  /**
   * The number of actions each player has.
   * The length of the array serves as a reference for the number of players.
   */
  private int []numActions;
  
  /**
   * The utility function. Maps arrays of actions to arrays of utilities.
   */
  private Utility utility;
  
  /**
   * Gets the utility function.
   * @return the utility function.
   */
  public Utility getUtility(){
    return utility;
  }
  
  /**
   * Gets the number of players in the game. Implemented
   * by checking the length of numActions.
   * @return the number of players
   */
  public int getNumPlayers(){
    return numActions.length;
  }
  
  
  
  public boolean areLegalActions(int[] actions){
    if (actions==null){
      return false;
    }
    if (actions.length!=numActions.length){
      return false;
    }
    for(int i=0;i<actions.length;i++){
      if ((actions[i]>=numActions[i]) || (actions[i]<0)){
        return false;
      }
    }
    return true;
  }
  /**
   * Gets the number of actions for a particular player.
   * @param playerIndex the player of interest
   * @return the number of actions available to player playerIndex.
   */
  public int getNumActions(int playerIndex){
    return numActions[playerIndex];
  }
  
  /**
   * Gets the number of actions for the first player. Each
   * player views herself as the first player.
   * @return the number of actions available to player 0.
   */
  public int getNumActions(){
    return numActions[0];
  }
  
  /**
   * Constructs a new game. The utility function should be
   * "immutable", in the sense that players cannot modify it.
   * @param numActions
   * @param utility
   */
  public Game(int[] numActions, Utility utility) {
    this.numActions = numActions;
    this.utility = utility;
  }
  
  public Game swapPlayers(int i, int j){
    if (utility.isSymmetric()){
      return this;
    }
    int[] newActions = new int[numActions.length];
    System.arraycopy(numActions, 0, newActions, 0, numActions.length);
    Array.swap(newActions,i,j);
    return new Game(newActions,utility.swapPlayers(i,j));
  }

}
