package results;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;



public class CalculateMeans {
  
  /**
   * Get the mean performances in a hashtable.
   * @param results
   * @return
   */
  public static Hashtable<String,Double> getMeans(List<? extends TripleResults> results){
    Hashtable<String,Double> totals = new Hashtable<String,Double>();
    for(TripleResults result:results){
      double[] means = result.getMeans();
      for(int i=0;i<result.playerNames.length;i++){
        String playerName = result.playerNames[i];
        double score = means[i];
        if (totals.containsKey(playerName)){
          totals.put(playerName, totals.get(playerName)+score);
        } else {
          totals.put(playerName, score);
        }
      }
    }
    return totals;
  }
  
  /**
   * Get the standard error of the mean.
   * @param results
   * @return
   */
  public static Hashtable<String,Double> getStandardErrorOfTheMean(List<? extends TripleResults> results){
    Hashtable<String,Double> totalVariances = new Hashtable<String,Double>();
    
    for(TripleResults result:results){
      double[] standardErrors = result.getStandardErrorOfTheMean();
      for(int i=0;i<result.playerNames.length;i++){
        String playerName = result.playerNames[i];
        double score = standardErrors[i] * standardErrors[i];
        if (totalVariances.containsKey(playerName)){
          totalVariances.put(playerName, totalVariances.get(playerName)+score);
        } else {
          totalVariances.put(playerName, score);
        }
      }
    }
    
    Hashtable<String,Double> standardErrorOfTheMean = new Hashtable<String,Double>();
    for(Map.Entry<String, Double> entry:totalVariances.entrySet()){
      standardErrorOfTheMean.put(entry.getKey(), Math.sqrt(entry.getValue()));
    }
    return standardErrorOfTheMean;
  }
  
  /**
   * Orders the players from best (0) to worst (array length-1).
   * @param results
   * @return
   */
  public static String[] getRankedNames(List<? extends TripleResults> results){
    Hashtable<String,Double> totals = getMeans(results);
    String[] rankedNames = new String[totals.size()];
    Vector<PlayerValue<Double >> sorted = PlayerValue.getSorted(totals);
    
    int index = rankedNames.length-1;
    for(PlayerValue<Double> total:sorted){
      rankedNames[index]=total.name;
      index--;
    }
    return rankedNames;
  }
  
  public static void showUsage(){
    System.err.println("Usage:java results.CalculateMeans <RESULTFILE> [<MAPPINGFILE> [REPEATS]?]?");
    System.err.println("Where <RESULTFILE> is the file generated by runTournament");
    System.err.println("<MAPPING> is a file with <CLASSNAME> <SPACE> <RESULTNAME> to pretty-print results");
    System.err.println("<REPEATS> is the number of times the results were repeated");
  }
  public static void main(String[] args) throws IOException{
    if (args.length==0){
      return;
    }
    File f = new File(args[0]);
    
    Hashtable<String,String> mapping = new Hashtable<String,String>();
    if (args.length>1){
      File mappingFile = new File(args[1]);
      TripleResults.loadHashtable(mappingFile,mapping);
    }
    
    int numRepeats = 100;
    if (args.length>2){
      numRepeats = Integer.parseInt(args[2]);
    }
    Vector<TripleResults> results;
    
    results = TripleResults.getResultsWithVariance(f,numRepeats);
    
    
    TripleResults.translate(results, mapping);
    Hashtable<String,Double> totals = getMeans(results);
    Vector<PlayerValue<Double> > sorted = PlayerValue.getSorted(totals);
    
    Hashtable<String,Double> sems = getStandardErrorOfTheMean(results);
    int gamesPerPlayer = results.size()*3/totals.size();
    int iterations = 100;
    int scalingFactor = gamesPerPlayer*iterations;
    Vector<String> finalresults = new Vector<String>();
    for(PlayerValue<Double> total:sorted){
      String player = total.name;
      String mean = String.format("%.4f", total.value/scalingFactor);
      String sem = String.format("%.4f", sems.get(player)/scalingFactor);
      finalresults.add(player+":"+mean+"+-"+sem);
    }
    for(int i=finalresults.size()-1;i>=0;i--){
      System.out.println(finalresults.get(i));
    }
    
  }
}
