package results;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;



public class CalculateTotals {
  
  
  public static Hashtable<String,Long> getScores(Vector<TripleResults> results){
    Hashtable<String,Long> totals = new Hashtable<String,Long>();
    for(TripleResults result:results){
      for(int i=0;i<result.playerNames.length;i++){
        String playerName = result.playerNames[i];
        long score = result.scores[i];
        if (totals.containsKey(playerName)){
          totals.put(playerName, totals.get(playerName)+score);
        } else {
          totals.put(playerName, score);
        }
      }
    }
    return totals;
  }
  
  public static Hashtable<String,Double> getStandardErrorOfTheMean(Vector<TripleResults> results){
    Hashtable<String,Double> totalVariances = new Hashtable<String,Double>();
    
    for(TripleResults result:results){
      double[] standardErrors = result.getStandardErrorOfTheMean();
      for(int i=0;i<result.playerNames.length;i++){
        String playerName = result.playerNames[i];
        double score = standardErrors[i] * standardErrors[i];
        if (totalVariances.containsKey(playerName)){
          totalVariances.put(playerName, totalVariances.get(playerName)+score);
        } else {
          totalVariances.put(playerName, score);
        }
      }
    }
    
    Hashtable<String,Double> standardErrorOfTheMean = new Hashtable<String,Double>();
    for(Map.Entry<String, Double> entry:totalVariances.entrySet()){
      standardErrorOfTheMean.put(entry.getKey(), Math.sqrt(entry.getValue()));
    }
    return standardErrorOfTheMean;
  }
  
  
  public static String[] getRankedNames(Vector<TripleResults> results){
    Hashtable<String,Long> totals = getScores(results);
    String[] rankedNames = new String[totals.size()];
    Vector<PlayerValue<Long>> sorted = PlayerValue.getSorted(totals);
    
    int index = rankedNames.length-1;
    for(PlayerValue<Long> value:sorted){
      rankedNames[index]=value.name;
      index--;
    }
    return rankedNames;
  }
  
  public static void main(String[] args) throws IOException{
    File f = new File(args[0]);
    
    Hashtable<String,String> mapping = new Hashtable<String,String>();
    if (args.length>1){
      File mappingFile = new File(args[1]);
      TripleResults.loadHashtable(mappingFile,mapping);
    }
    Vector<TripleResults> results;
    
    if (args.length>2 && args[2].equals("-variance")){
      results = TripleResults.getResultsWithVariance(f,100);
    } else {
      results = TripleResults.getResults(f);
    }
    
    
    TripleResults.translate(results, mapping);
    Hashtable<String,Long> totals = getScores(results);
    System.out.println("Original order");
    for(Map.Entry<String,Long> total:totals.entrySet()){
      System.out.println(total.getKey()+":"+total.getValue());
    }
    Vector<PlayerValue<Long>> sorted = PlayerValue.getSorted(totals);
    
    System.out.println("Sorted order");
    
    for(PlayerValue<?> total:sorted){
      System.out.println(total.value+":"+total.name);
    }
    
  }
}
