package results;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


public class KnockoutTable {
  public static void removePlayer(Vector<TripleResults> results, String name){
    Iterator<TripleResults> iterator = results.iterator();
    while(iterator.hasNext()){
      TripleResults current = iterator.next();
      if (current.getIndex(name)!=-1){
        iterator.remove();
      }
    }
  }

  public static void displayRanking(String[] names){
    for(int i=0;i<names.length;i++){
      System.out.println(""+(i+1)+":"+names[i]);
    }
    
  }
  
  
  public static void displayRanking(String[] names,List<? extends TripleResults> matchData){
    Hashtable<String,Double> stderr = CalculateMeans.getStandardErrorOfTheMean(matchData);
    Hashtable<String,Double> mean = CalculateMeans.getMeans(matchData);
    int gamesPerPlayer = matchData.size()*3/names.length;
    int iterations = 100;
    int scalingFactor = gamesPerPlayer*iterations;
    double previousMean = 0;
    double previousError = 0;
    for(int i=0;i<names.length;i++){
      String place = ""+(i+1);
      String playerName = names[i];
      double currentError = stderr.get(playerName)/scalingFactor;
      String thisError = String.format("%.4f", currentError);
      double currentMean = mean.get(playerName)/scalingFactor;
      String thisMean = String.format("%.4f", currentMean);
      if (previousMean!=0.0){
        String meanDifference = String.format("%.4f", (previousMean-currentMean));
        String totalError = String.format("%.4f",previousError+currentError );
        System.out.println("(difference:"+meanDifference+"+-"+totalError+")");
      }
      System.out.println(place+":"+names[i]+" "+thisMean+"+-"+thisError);
      
      previousMean = currentMean;
      previousError = currentError;
    }
    
  }
  public static String[] getKnockoutOrder(List<? extends TripleResults> matchData){
    Vector<TripleResults> temp = new Vector<TripleResults>(matchData);
    
    String[] currentOrder = CalculateMeans.getRankedNames(temp);
    System.out.println("Initial:");
    displayRanking(currentOrder,temp);
    String[] ranking = new String[currentOrder.length];
    while(currentOrder.length>3){
      String knockout = currentOrder[currentOrder.length-1];
      ranking[currentOrder.length-1]=knockout;
      removePlayer(temp,knockout);
      System.out.println("After "+(1+ranking.length-currentOrder.length)+" removed:");
      currentOrder = CalculateMeans.getRankedNames(temp);
      displayRanking(currentOrder,temp);
    }
    System.arraycopy(currentOrder,0,ranking,0,3);
    return ranking;
  }
  
  
  
  public static void main(String[] args) throws IOException{
    File file = new File(args[0]);
    File mappingFile = new File(args[1]);
    int numRepeats = Integer.parseInt(args[2]);
    Vector<TripleResults> results = TripleResults.getResultsWithVariance(file,numRepeats);
    Hashtable<String,String> mapping = new Hashtable<String,String>();
    TripleResults.loadHashtable(mappingFile, mapping);
    TripleResults.translate(results, mapping);
    String[] knockoutOrder = getKnockoutOrder(results);
    System.out.println("Final ranking:");
    displayRanking(knockoutOrder);
    
  }
}
