package stats.collusion;

import game.Game;
import game.Utility;

public class FullGameCollusionObserver extends AbstractCollusionObserver {

  int wholeGameCollusionCount;
  boolean beganEarly;
  
  int[] utilityInCollusion;
  
  int[] totalUtilityInCollusion;
  int[] totalRoundsInCollusion;
  
  
  Utility util;
  public FullGameCollusionObserver(Game g){
    super(g);
    util = g.getUtility();
    wholeGameCollusionCount=0;
    beganEarly = false;
    utilityInCollusion = null;
    totalUtilityInCollusion = new int[g.getNumPlayers()];
    totalRoundsInCollusion = new int[g.getNumPlayers()];
  }
  
  @Override
  public void observeContinueCollusion() {
    incrementUtility();
  }

  private void incrementUtility(){
    int[] currentUtil = util.getUtility(actions);
    
    if (utilityInCollusion==null){
      utilityInCollusion = new int[currentUtil.length];
    }
    for(int i=0;i<currentUtil.length;i++){
      utilityInCollusion[i]+=currentUtil[i];
    }
    
  }
  static final int END_OF_EARLY=10;
  @Override
  public void observeStartCollusion() {
    if (this.currentRound % 100<=END_OF_EARLY){
      beganEarly = true;
    }
    incrementUtility();
    
  }

  int betterColluder;
  int worseColluder;
  @Override
  public void observeTerminateCollusion(boolean countBroken) {
    if (!countBroken && beganEarly){
      wholeGameCollusionCount++;
      
      int player0 = this.collusionPlayers[0];
      int player1 = this.collusionPlayers[1];
      int util0 = this.utilityInCollusion[player0];
      int util1 = this.utilityInCollusion[player1];
      if (util0>util1){
        betterColluder+=util0;
        worseColluder+=util1;
      } else {
        betterColluder +=util1;
        worseColluder +=util0;
      }
      int roundsInCollusion = (util0+util1)/18;
      totalRoundsInCollusion[player0]+=roundsInCollusion;
      totalRoundsInCollusion[player1]+=roundsInCollusion;
      totalUtilityInCollusion[player0]+=utilityInCollusion[player0];
      totalUtilityInCollusion[player1]+=utilityInCollusion[player1];
    }
    
    beganEarly = false;
    utilityInCollusion = null;
  }

  
  /**
   * Displays the data in JSON format.
   * Whole game collusions: number of games where there was collusion between the 10th and 100th round.
   * Lower colluders average: average utility of the lower utility colluder in each game.
   * data: player specific data
   * Player: player class
   * averageUtility: average utility in whole game collusions
   * Rounds: number of rounds in whole game collusions
   * totalUtility: total utility in collusions.
   */
  @Override
  public void display(String[] playerNames) {
    System.out.println("{\"Whole game collusions\":"+wholeGameCollusionCount+",");
    int totalWon = betterColluder+worseColluder;
    int rounds = totalWon/18;
    double lowerAverage = ((double)worseColluder)/rounds;
    if (rounds==0){
      lowerAverage = 0;
    }
    System.out.println("\"Lower colluders average\":"+lowerAverage+",");
    System.out.print("\"data\":[");
    for(int i=0;i<3;i++){
      final double averageUtility;
      if (totalRoundsInCollusion[i]==0){
        averageUtility = 0;
      } else {
        averageUtility = ((double)totalUtilityInCollusion[i]/totalRoundsInCollusion[i]);
      }
      String data = "\"Player\":\""+playerNames[i]+"\",\"totalUtility\":"+totalUtilityInCollusion[i]+"\",\"averageUtility\":"+averageUtility+ ",\"rounds\":"+totalRoundsInCollusion[i];
      if (i!=2){
        System.out.println(data+",");
      } else {
        System.out.println(data+"]");
      }
    }
    System.out.println("}");
    
  }

  @Override
  public void startGame(Game g) {
    // TODO Auto-generated method stub
    
  }

}
