package stats.collusion;

import game.Game;


/**
 * An observer of utility, like jointactionutility, that does
 * not keep track of joint actions, simply utilities.
 * @author maz
 *
 */
public class PairwiseCollusionObserver extends AbstractCollusionObserver{

  int[][] collusionsStarted;
  int[][] collusionsBroken;
  int[][] totalCollusionLength;
  int[][][] collusionBreaker;
  
  public void observeStartCollusion(){
    int player0 = this.collusionPlayers[0];
    int player1 = this.collusionPlayers[1];
    this.collusionsStarted[player0][player1]++;
    this.totalCollusionLength[player0][player1]++;
  }
  
  
  
  public PairwiseCollusionObserver(Game g){
    this(g.getNumPlayers());
  }
  
  
  /**
   * Construct a utility observer.
   * @param utility a utility function to be used.
   * @param numPlayers the number of players in the game.
   */
  public PairwiseCollusionObserver(int numPlayers){
    this.collusionsStarted = new int[numPlayers][numPlayers];
    this.collusionsBroken = new int[numPlayers][numPlayers];
    this.totalCollusionLength = new int[numPlayers][numPlayers];
    this.collusionBreaker = new int[numPlayers][numPlayers][numPlayers+1];
    AbstractCollusionObserver.MIN_LENGTH = 1;
  }
  
  
  
  public void observeTerminateCollusion(boolean countBroken){
    int player0 = this.collusionPlayers[0];
    int player1 = this.collusionPlayers[1];

    int collusionLength = this.currentRound - this.roundCollusionStarted;
    if (this.collusionDidNotCount()){
      // A one-round collusion doesn't count
      collusionsStarted[player0][player1]--;
      totalCollusionLength[player0][player1]-=collusionLength;
    } else if (countBroken){
      collusionsBroken[player0][player1]++;
      boolean player0Breaks = (this.collusionActions[player0]!=this.actions[player0]);
      boolean player1Breaks = (this.collusionActions[player1]!=this.actions[player1]);
      if (player0Breaks && (!player1Breaks)){
        collusionBreaker[player0][player1][player0]++;
      }
      if (player1Breaks && (!player0Breaks)){
        collusionBreaker[player0][player1][player1]++;
      }
      if (player0Breaks && (player1Breaks)){
        collusionBreaker[player0][player1][3]++;
      }
    }
  }
  
  public void observeContinueCollusion(){
    int player0 = this.collusionPlayers[0];
    int player1 = this.collusionPlayers[1];

    this.totalCollusionLength[player0][player1]++;
  }
  
  
  
  public int[][] getCollusionsStarted(){
    return collusionsStarted;
  }
  public int[][] getCollusionsBroken(){
    return collusionsBroken;
  }
  
  public int[][] getTotalCollusionLength(){
    return totalCollusionLength;
  }

  public static String getString(int[] counts){
    StringBuilder builder = new StringBuilder("[");
    if (counts.length>=1){
      builder.append(counts[0]);
    }
    
    for(int j=1;j<counts.length;j++){
      builder.append(","+counts[j]);
    }
    builder.append("]");
    return builder.toString();
  }

  public static String getString(int[][] counts){
    StringBuilder builder = new StringBuilder("[");
    if (counts.length>=1){
      builder.append(getString(counts[0]));
    }
    for(int j=1;j<counts.length;j++){
      builder.append(","+getString(counts[j]));
    }
    builder.append("]");
    return builder.toString();
  }

  public static String getString(int[][][] counts){
    StringBuilder builder = new StringBuilder("[");
    if (counts.length>=1){
      builder.append(getString(counts[0]));
    }
    for(int j=1;j<counts.length;j++){
      builder.append(","+getString(counts[j]));
    }
    builder.append("]");
    return builder.toString();
  }
  
  /**
   * Display the utilities for the series and the squares of the utilities.
   */
  public void display(String[] playerNames) {
    System.out.print("{");
    System.out.print("\"all\":[" + playerNames[0]+","+playerNames[1]+","+playerNames[2]+"],");
    System.out.print("\"collusionsStarted\":"+getString(collusionsStarted)+",");
    System.out.print("\"collusionsBroken\":"+getString(collusionsBroken)+",");
    System.out.print("\"totalCollusionLength\":"+getString(totalCollusionLength)+",");
    System.out.println("\"collusionBreaker\":"+getString(collusionBreaker)+"}");
  }



  @Override
  public void startGame(Game g) {
    // TODO Auto-generated method stub
    
  }
  


}
