package stats.collusion;

import java.util.Arrays;

import stats.SeriesObserver;

import game.Game;
import game.Utility;


/**
 * An observer of utility, like jointactionutility, that does
 * not keep track of joint actions, simply utilities.
 * @author maz
 *
 */
public class SimpleCollusionObserver implements SeriesObserver{
  
  int collusionLength;
  int notCollusionLength;
  
  boolean gameHasCollusion;
  int gamesWithCollusion;
  int[] utilityInCollusion;
  int[] utilityNotInCollusion;
  
  
  public boolean isInCollusion(int actionA, int actionB){
    return ((actionA+ 6) % 12==actionB);
  }
  
  
  public boolean testCollusion(int[] actions){
    for(int i=0;i<actions.length;i++){
      for(int j=i+1;j<actions.length;j++){
        if (isInCollusion(actions[i],actions[j])){
          return true;
        }
      }
    }
    return false;
  }
  
  Utility utility;
  
  public SimpleCollusionObserver(Game g){
    gameHasCollusion = false;
    utility = g.getUtility();
    this.utilityInCollusion = new int[g.getNumPlayers()];
    this.utilityNotInCollusion = new int[g.getNumPlayers()];
  }
  
  
  
  public static void increment(int[] a, int[]increment){
    for(int i=0;i<a.length;i++){
      a[i]+=increment[i];
    }
  }
  public void observeOutcome(int[] actions) {
    int []util = utility.getUtility(actions);
    if (testCollusion(actions)){
      increment(this.utilityInCollusion,util);
      this.collusionLength++;
      if (!gameHasCollusion){
        this.gamesWithCollusion++;
        gameHasCollusion=true;
      }
      
    } else {
      increment(this.utilityNotInCollusion,util);
      notCollusionLength++;
    }
  }
  
  
  
  

  public void finishGame() {
    gameHasCollusion = false;
  }

  public void finishSeries() {
  }


  @Override
  public void display(String[] playerNames) {
    System.out.print("{\"playerNames\":["+playerNames[0]+","+playerNames[1]+","+playerNames[2]+"],");
    System.out.print("\"gamesWithCollusion\":"+gamesWithCollusion+",");
    System.out.print("\"utilityInCollusion\":"+Arrays.toString(utilityInCollusion)+",");
    System.out.print("\"utilityNotInCollusion\":"+Arrays.toString(utilityNotInCollusion)+",");
    System.out.print("\"notCollusionLength\":"+notCollusionLength+",");    
    System.out.println("\"collusionLength\":"+collusionLength+"}");    
  }


  @Override
  public void startGame(Game g) {
    // TODO Auto-generated method stub
    
  }
}
