package strategy;

import game.Game;
import game.Strategy;

import java.util.Random;



/**
 * A strategy that chooses a random action on the first round, and then
 * plays that action thereafter.
 * @author maz
 *
 */
public class ConstantStrategy implements Strategy{
  /**
   * The action the player plays
   */
  int constantAction;
  
  
  /**
   * Construct a constant strategy. The game is
   * used to get the number of actions, and the
   * seed is used to seed a pseudorandom number 
   * generator, which then is queried for the action 
   * to take.
   * 
   * @param g
   * @param seed
   */
  public ConstantStrategy(Game g, long seed, String options) {
    int numActions = g.getNumActions();
    Random r = new Random(seed);
    constantAction = r.nextInt(numActions);
  }
  



  public int getAction() {
    return constantAction;
  }

  public void observeOutcome(int[] actions) {
  }
  
  
  
}
