package com.yahoo.lsg.stats.generalized;

import static org.junit.Assert.*;


import game.Game;
import game.GameGenerator;
import lemonade.GeneralizedLemonadeGameGenerator;
import lemonade.GeneralizedLemonadeUtility;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class CollusionCountTest {
  GameGenerator generator;
  CollusionCount counter;
  @Before
  public void setUp() throws Exception {
    generator = new GeneralizedLemonadeGameGenerator(12);
    counter = new CollusionCount(generator);
    GeneralizedLemonadeUtility glu = new GeneralizedLemonadeUtility(
        new int[]{6,6,6,6,6,
                  6,6,6,6,6,
                  6,6});
    
    counter.startGame(new Game(new int[]{12,12,12},glu));
  }

  @After
  public void tearDown() throws Exception {
    counter = null;
    generator = null;
  }

  
  @Test
  public void testImpossibleToCollude(){
    //int countImpossible = 0;
    for(int i=0;i<10000;i++){
      counter.startGame(generator.getNextGame());
      if (counter.currentHasNoCollusions){
        return;
      }
    }
    fail("Would expect at least one game without collusions (more like 18)");
  }
  
  @Test
  public void testToString() {
    counter.observeOutcome(new int[]{0,5,9});
    String str = counter.toString();
    assertEquals("{\"numCollusions\":0,\"numRounds\":1,\"impossibleToCollude\":0}",str);
  }


  @Test
  public void testToStringArray() {
    counter.observeOutcome(new int[]{0,5,9});
    String str = counter.toString(new String[]{"A","B","C"});
    assertEquals("{\"numCollusions\":0,\"numRounds\":1,\"impossibleToCollude\":0,\"playerNames\":[\"A\", \"B\", \"C\"]}",str);
  }

  @Test
  public void testIsCollusion() {
    boolean opposite = counter.isCollusion(0,6);
    assertTrue(opposite);
  }

  @Test
  public void testIsCollusion2() {
    boolean adjacent = counter.isCollusion(0,1);
    assertFalse(adjacent);
  }

  @Test
  public void testIsCollusion3() {
    boolean result = counter.isCollusion(0,5);
    assertFalse(result);
  }

  @Test
  public void testHasCollusion01() {
    boolean result = counter.hasCollusion(new int[]{0,6,9});
    assertTrue(result);
  }

  @Test
  public void testHasCollusion02() {
    boolean result = counter.hasCollusion(new int[]{0,9,6});
    assertTrue(result);
  }


  @Test
  public void testHasCollusion12() {
    boolean result = counter.hasCollusion(new int[]{9,0,6});
    assertTrue(result);
  }

  @Test
  public void testHasCollusionNot() {
    boolean result = counter.hasCollusion(new int[]{0,5,9});
    assertFalse(result);
  }

  @Test
  public void testObserveOutcome() {
    counter.observeOutcome(new int[]{9,0,6});
    assertEquals(1, counter.numCollusions);
    assertEquals(1, counter.numRounds);
  }
  
  @Test
  public void testObserveOutcome2() {
    counter.observeOutcome(new int[]{0,1,2});
    assertEquals(0, counter.numCollusions);
    assertEquals(1, counter.numRounds);
  }
  
  @Test
  public void testObserveOutcome3() {
    counter.observeOutcome(new int[]{9,0,6});
    counter.observeOutcome(new int[]{9,0,6});
    assertEquals(2, counter.numCollusions);
    assertEquals(2, counter.numRounds);
  }
  
  @Test
  public void testObserveOutcome4() {
    counter.observeOutcome(new int[]{0,5,9});
    assertEquals(0, counter.numCollusions);
    assertEquals(1, counter.numRounds);
  }
  

  
}
