package results;

import static org.junit.Assert.*;

import java.io.File;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class AlternativeCompetitionsFastTest {
  AlternativeCompetitionsFast competitions;
  EmpiricalGame tournament;
  @Before
  public void setUp() throws Exception {
    tournament = EmpiricalGame.getAsymmetricGame(TripleResults.getResultsWithVariance(new File("results/metaOnlyGame.txt"),100),new String[]{"rlcubed.CoOpp","edu.cmu.psy.reitter.Meta", "strategy.ModifiedConstant"});
    competitions = new AlternativeCompetitionsFast(tournament,new String[]{"rlcubed.CoOpp","edu.cmu.psy.reitter.Meta", "strategy.ModifiedConstant"},9);
  }

  @After
  public void tearDown() throws Exception {
    competitions = null;
    tournament = null;
  }

  @Test
  public void testAlternativeCompetitions() {
    assertNotNull(competitions);
    
  }

  @Test
  public void testGetExpectedUtility() {
    competitions.draws = 100;
    double value0 = competitions.getExpectedUtility(0, 0);
    double value1 = competitions.getExpectedUtility(0, 1);
    double value2 = competitions.getExpectedUtility(0, 2);
    
    assertEquals(0.176,value0,0.01);
    
    assertEquals(0.113,value1,0.01);
    assertEquals(0.0,value2,0.01);
  }

  @Test
  public void testGetExpectedUtility2() {
    competitions = new AlternativeCompetitionsFast(tournament,new String[]{"rlcubed.CoOpp","edu.cmu.psy.reitter.Meta", "strategy.ModifiedConstant"},3);
    competitions.draws = 100;
    competitions.weights=new double[][]{
        {1.0,0.0,0.0},
        {0.0,1.0,0.0},
        {0.0,0.0,1.0}
    };
    
    double value0 = competitions.getExpectedUtility(0, 0);
    double value1 = competitions.getExpectedUtility(0, 1);
    double value2 = competitions.getExpectedUtility(0, 2);
    
    assertEquals(1.0,value0,0.01);
    
    assertEquals(0.5,value1,0.01);
    assertEquals(0.0,value2,0.01);
  }

  @Test
  public void testGetExpectedUtility3() {
    competitions = new AlternativeCompetitionsFast(tournament,new String[]{"rlcubed.CoOpp","edu.cmu.psy.reitter.Meta", "strategy.ModifiedConstant"},9);
    competitions.draws = 100;
    competitions.weights=new double[][]{
        {0.65,0.0,0.35},
        {0.65,0.0,0.35},
        {0.65,0.0,0.35},
        {0.65,0.0,0.35},
        {0.65,0.0,0.35},
        {0.65,0.0,0.35},
        {0.65,0.0,0.35},
        {0.65,0.0,0.35},
        {0.65,0.0,0.35}
    };
    
    double value0 = competitions.getExpectedUtility(0, 0);
    double value1 = competitions.getExpectedUtility(0, 1);
    double value2 = competitions.getExpectedUtility(0, 2);
    
    assertEquals(0.108,value0,0.1);
    
    assertEquals(0.88,value1,0.1);
    assertEquals(0.09,value2,0.01);
  }

  @Test
  public void testGetUtilityDraw() {
    competitions = new AlternativeCompetitionsFast(tournament,new String[]{"rlcubed.CoOpp","edu.cmu.psy.reitter.Meta", "strategy.ModifiedConstant"},3);
    competitions.draws = 100;
    competitions.weights=new double[][]{
        {1.0,0.0,0.0},
        {0.0,1.0,0.0},
        {0.0,0.0,1.0}
    };
    
    double value0 = competitions.getExpectedUtility(0, 0);
    
    assertEquals(1.0,value0,0.01);
    
  }

  @Test
  public void testGetUtility() {
    competitions = new AlternativeCompetitionsFast(tournament,new String[]{"rlcubed.CoOpp","edu.cmu.psy.reitter.Meta", "strategy.ModifiedConstant"},3);
    double utility = competitions.getUtility(0,new int[]{0,1, 2});
    assertEquals(1.0,utility,0.001);
  }

  @Test
  public void testGetUtility3() {
    double utility = competitions.getUtility(0,new int[]{
        1, 
        0,
        0,
        0,
        0,
        2,
        2,
        2,
        2
    });
    assertEquals(1.0,utility,0.001);
  }

  @Test
  public void testGetUtility4() {
    double utility = competitions.getUtility(0,new int[]{
        1, 
        0,
        0,
        0,
        2,
        2,
        2,
        2,
        2
    });
    assertEquals(0.0,utility,0.001);
  }


  @Test
  public void testGetUtility5() {
    double utility = competitions.getUtility(0,new int[]{
        1, 
        0,
        2,
        2,
        2,
        2,
        2,
        2,
        2
    });
    assertEquals(0.0,utility,0.001);
  }


  @Test
  public void testGetUtility6() {
    double utility = competitions.getUtility(1,new int[]{
        1,
        0,
        2,
        2,
        2,
        2,
        2,
        2,
        2
    });
    assertEquals(1.0,utility,0.001);
  }


  @Test
  public void testGetUtility2() {
    double utility = competitions.getUtility(0,new int[]{
        1, 
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0
    });
    assertEquals(1.0,utility,0.001);
  }

  @Test
  public void testGetInitialDist() {
    String map = "[rlcubed.CoOpp=0.65,strategy.ModifiedConstant=0.35]";
    double []dist = competitions.getInitialDist(map);
    assertEquals(0.65,dist[0],0.01);
    assertEquals(0.0,dist[1],0.01);
    assertEquals(0.35,dist[2],0.01);
  }

}
