package org.zinkevich.lsg.wrapper;

import game.Game;
import game.Strategy;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Random;

import com.yahoo.lsg.lifelong.GameInformation;
import com.yahoo.lsg.lifelong.LifelongStrategy;

public class Wrapper implements LifelongStrategy{
  Strategy strategy;
  
  Random seedGenerator;
  String strategyClassName;
  Constructor<?> strategyConstructor;
  
  public void resetStrategy(Game g){
	    Class<?>[] params = new Class<?>[]{
	      game.Game.class,
	      long.class,String.class};
	    String paramString = "(game.Game,long,java.lang.String)";
	    try{
	    Class<?> c = Class.forName(strategyClassName);
	    String options = "";
	    Constructor<?> constr = c.getConstructor(params);
	    strategy = (Strategy)(constr.newInstance(g,seedGenerator.nextLong(),options));
	        } catch(ClassNotFoundException cnf){
	          System.err.println("Could not find "+strategyClassName);
	          throw new RuntimeException(cnf);
	        } catch(SecurityException s){
	          System.err.println("No permission to access "+strategyClassName+paramString);
	          throw new RuntimeException(s);
	        } catch (NoSuchMethodException nsm){
	          System.err.println("Could not find "+strategyClassName+paramString);
	          throw new RuntimeException(nsm);
	        } catch (InstantiationException ie){
	          System.err.println("Cannot instantiate "+strategyClassName);
	          throw new RuntimeException(ie);
	        } catch (InvocationTargetException ite){
	          System.err.println("Constructor "+strategyClassName+paramString+" threw an exception");
	          throw new RuntimeException(ite);
	        } catch (IllegalArgumentException e) {
	          throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
			  throw new RuntimeException(e);
			}

  }
  
  public Wrapper(GameInformation info, long seed, String options){
	strategyClassName = options;
    seedGenerator = new Random(seed);
    
  }
  
  @Override
  public int getAction() {
    return strategy.getAction();
  }
  
  @Override
  public void newGame(Game g, int iterations) {
	resetStrategy(g);
  }
  
  @Override
  public void observeOutcome(int[] actions) {
	strategy.observeOutcome(actions);
  }
  
}
